@@ -7,14 +7,16 @@ Usage: ./configure.sh <option>
 
 Available options: 
 	help		this screen
-	linux		to compile on a linux host
+	linux		to compile on a linux host and install in /usr/local
+	linux-slack	to compile on a linux slack box (installs in /usr)
 	osx		to compile on a darwin host
 	osx-static	to compile on a darwin host with some static libs
 	obsd		to compile on a openbsd host
 	fbsd 		to compile on a freebsd host
 	beos		to compile on a beos host
 	cygwin		to compile on a cygwin environment
-	win		to cross-compile for win on a linux host with mingw32msvc
+	win		to cross-compile for win on a linux host with 
+			mingw32msvc (read BUILD for more info)
 
 EOT
 
@@ -41,6 +43,7 @@ HLDFLAGS=$LDFLAGS
 WINDRES=windres
 DLLTOOL=dlltool
 MAKE=make
+WHERE=/usr/local/
 }
 
 set_linux() {
@@ -48,28 +51,36 @@ set_default
 OS=Linux
 }
 
+set_linux_slack() {
+set_default
+CFLAGS="-O2 -g3 -march=i486 -Wall -DHAVE_CONFIG_H -I$PWD"
+WHERE=/usr/
+OS=Linux
+}
+
 set_osx() {
 set_default
 OS=Darwin
 CFLAGS="$CFLAGS -I/sw/include -DMACOSX"
 HCFLAGS="$HCFLAGS -I/sw/include -DMACOSX"
-LDFLAGS="$LDFLAGS -L/usr/lib -L/sw/lib -bind_at_load -framework Carbon"
-HLDFLAGS="$HLDFLAGS -L/usr/lib -L/sw/lib -bind_at_load"
+LDFLAGS="$LDFLAGS -bind_at_load -framework Carbon"
+HLDFLAGS="$HLDFLAGS -bind_at_load"
 }
 
 set_osx_static() {
 set_default
 OS=Darwin-static
-CFLAGS="$CFLAGS -DMACOSX"
-HCFLAGS="$HCFLAGS -DMACOSX"
-LDFLAGS="$LDFLAGS -L/usr/lib -bind_at_load -framework Carbon"
-HLDFLAGS="$HLDFLAGS -L/usr/lib -bind_at_load"
+CFLAGS="$CFLAGS -I/sw/include -DMACOSX"
+HCFLAGS="$HCFLAGS -I/sw/include -DMACOSX"
+LDFLAGS="$LDFLAGS -bind_at_load -framework Carbon"
+HLDFLAGS="$HLDFLAGS -bind_at_load"
 }
 
 set_fbsd() {
 set_default
 OS=FreeBSD
-CFLAGS="$CFLAGS -DMACOSX -DFREEBSD"
+CFLAGS="$CFLAGS -DMACOSX -DFREEBSD -I/usr/local/include"
+LDFLAGS="$LDFLAGS -L/usr/local/lib" 
 MAKE=gmake
 }
 
@@ -83,9 +94,9 @@ MAKE=gmake
 set_beos() {
 set_default
 OS=BeOS
-CFLAGS="$CFLAGS -DBEOS -I/boot/home/config/include/ -I/boot/develop/headers/be/bone/"
+CFLAGS="$CFLAGS -DBEOS -I/boot/home/config/include/ "
 LDFLAGS="$LDFLAGS -L/boot/home/config/lib/"
-HCFLAGS="$HCFLAGS -DBEOS -I/boot/home/config/include/ -I/boot/develop/headers/be/bone/"
+HCFLAGS="$HCFLAGS -DBEOS -I/boot/home/config/include/ "
 HLDFLAGS="$HLDFLAGS -L/boot/home/config/lib/"
 
 }
@@ -141,13 +152,15 @@ case $1 in
 	linux)
 		set_linux
 	;;
+	linux-slack)
+		set_linux_slack
+	;;
 	obsd)
 		set_obsd
 	;;
 	fbsd)
 		set_fbsd
 	;;
-
 	osx)
 		set_osx
 	;;
@@ -191,5 +204,6 @@ WINDRES=$WINDRES
 DLLTOOL=$DLLTOOL
 OS=$OS
 MAKE=$MAKE
+WHERE=$WHERE
 EOT
 
