@@ -6,9 +6,11 @@
 -- "http://" form,exhed are extra header lines you want to add
 -- , for example {"Range: bytes 0-100","User-agent: fake" }<BR/>
 -- <BR/>
--- <B>get_head(uri,exhed)</B> : returns string,err and takes the uri in 
+-- <B>get_head(uri,exhed,fallback)</B> : returns string,err and takes 
+-- the uri in 
 -- "http://" form,exhed are extra header lines you want to add. 
--- returns only the header, not the the body<BR/>
+-- returns only the header, not the the body. If fallback is true then
+-- a GET with range: bytes 0 is tryed<BR/>
 -- <BR/>
 -- <B>pipe_uri(uri,callback,exhed)</B> : 
 -- Gets the uri and uses callback on the data
@@ -20,7 +22,12 @@
 -- on error, "" on end of transmission. 
 -- It also return an error if one.<BR>
 -- See
--- libero.lua for an example on how to use the callback.<BR>
+-- libero.lua for an example on how to use the callback.<BR/>
+-- <BR/>
+-- <B>pipe_uri_with_header(self,url,cb_h,cb_b,exhed)</B> :
+-- As pipe uri, but uses cb_h for the header and cb_b fpor the body.<BR/>
+-- Since the browser module doesn't know the result of the GET it 
+-- will not follow redirects. The mimer module uses this.<BR/>
 -- <BR/>
 -- <B>post_uri(uri,post,exhed)</B> : returns string,err and takes the uri in 
 -- "http://" form, the post data in "name=val&..." form 
@@ -164,11 +171,80 @@ function Hidden.cookie_and_referer(self,url,gl_h)
 
 end
 
+-- adds dirname(u.path) .. / .. location if needed
+function Hidden.adjust_path(l,u,location)
+	local function clean_2_slash(s)
+		return (string.gsub(s,"//","/"))
+	end
+	local function dirname(path)
+		local base = ""
+		if string.sub(path, -1, -1) == "/" then
+			-- is a dir, so the dirname is the whole path
+			if (string.sub(path, 1, 1) ~= "/") then
+				base = "/" .. path
+			else 
+				base = path
+			end
+		else
+			local rc = {}
+			string.gsub(path,"([^/]+)",
+				function(s)table.insert(rc,s)
+			end)
+			-- delete last element
+			table.remove(rc,table.getn(rc))
+			base = "/" .. table.concat(rc,"/") .. "/"
+		end
+		return clean_2_slash(base)
+	end
+
+	local u_dir = dirname(u.path)
+	local l_dir = dirname(l.path)
+
+	if (l_dir == "/") then 
+		return clean_2_slash(u_dir .. location)
+	else
+		local x,y = string.find(u_dir,l_dir)
+		if ( x == nil or y ~= string.len(u_dir)) then
+			-- the l_dir path is not included in the u_dir, so
+			-- we keep it untouched
+			return clean_2_slash("/" .. location)
+		else
+			return clean_2_slash(
+				string.sub(u_dir,1,x) .. "/" .. location)
+		end
+	end
+	
+end
+
 -- gets the field Location: in a header table
-function Hidden.get_location(gl_h)
+function Hidden.get_location(gl_h,url)
 	return table.foreach(gl_h,function(_,l)
 		local _,_,location = string.find(l,
 			"[Ll][Oo][Cc][Aa][Tt][Ii][Oo][Nn]%s*:%s*([^\r\n]*)")
+		if location ~= nil then
+			-- ah ah ah, what do you think? the RFC says that
+			-- Location wants an absolute uri, but the 
+			-- wounderful IIS sends a relative uri
+			local l = cookie.parse_url(location)
+			if ( l.host == nil or l.scheme == nil) then
+				local u = cookie.parse_url(url)
+				if ( u.host == nil or u.scheme == nil) then
+					error("get_location must be called "..
+						"with an absolute uri")
+				end
+				location = Hidden.adjust_path(l,u,location)
+				if (l.host == nil) then
+					location = u.host .. location
+				end
+				if (l.scheme == nil) then
+					location = u.scheme .. "://"..location
+				end
+				l = cookie.parse_url(location)
+				if ( l.host == nil or l.scheme == nil) then
+					error("unable to recover bad Location")
+				end
+			end
+		end
 		return location
 	end)
 end
@@ -197,10 +273,13 @@ end
 -- nil,error if an error
 -- DONE,nil if ok
 -- REDO,url if 3xx code
-function Hidden.parse_header(self,gl_h)
+function Hidden.parse_header(self,gl_h,url)
+	if gl_h[1] == nil then
+		return Hidden.error("malformed HTTP header line: nil")
+	end
 	local _,_,ret = string.find(gl_h[1],"[^%s]+%s+(%d%d%d)")
 	if ret == nil then
-		print("STRANGE HEADER!")
+		--print("STRANGE HEADER!")
 		table.foreach(gl_h,print)
 		return Hidden.error("malformed HTTP header line: "..gl_h[1])
 	end
@@ -217,7 +296,7 @@ function Hidden.parse_header(self,gl_h)
 				end_of_1xx = true
 			end
 		end
-		return Hidden.parse_header(self,gl_h1)
+		return Hidden.parse_header(self,gl_h1,url)
 	-- HTTP 2xx
 	elseif string.byte(ret,1) == string.byte("2",1) then
 		if self.followRefreshHeader == true then
@@ -233,7 +312,7 @@ function Hidden.parse_header(self,gl_h)
 			return Hidden.error("Unsupported HTTP "..ret.." code")
 		end
 		if ret=="301" or ret=="302" or ret=="303" or ret=="307"  then
-			local l = Hidden.get_location(gl_h)
+			local l = Hidden.get_location(gl_h,url)
 			if l ~= nil then
 				return Hidden.REDO,l
 			else
@@ -253,7 +332,7 @@ end
 
 -- starts curl!
 function Hidden.perform(self,url,gl_h,gl_b)
-	-- the callbacks
+	-- the callback for the body
 	if type(gl_b) == "table" then
 		self.curl:setopt(curl.OPT_WRITEFUNCTION,Hidden.build_w_cb(gl_b))
 	elseif type(gl_b) == "function" then
@@ -262,13 +341,28 @@ function Hidden.perform(self,url,gl_h,gl_b)
 		error("Hidden.perform must be called with table/function gl_b"..
 			", but is called with a "..type(gl_b))		
 	end
-	self.curl:setopt(curl.OPT_HEADERFUNCTION,Hidden.build_w_cb(gl_h))
+	-- the callback for the header
+	if type(gl_h) == "table" then
+		self.curl:setopt(
+			curl.OPT_HEADERFUNCTION,Hidden.build_w_cb(gl_h))
+	elseif type(gl_h) == "function" then
+		self.curl:setopt(curl.OPT_HEADERFUNCTION,gl_h)
+	else
+		error("Hidden.perform must be called with table/function gl_h"..
+			", but is called with a "..type(gl_h))		
+	end
+
 	local rc,err = self.curl:perform()
 
 	-- check result
 	if rc == 0 then
-		Hidden.cookie_and_referer(self,url,gl_h)
-		return Hidden.parse_header(self,gl_h)
+		if type(gl_h) == "function" then
+			-- we haven *not* the header!
+			return rc,err
+		else
+			Hidden.cookie_and_referer(self,url,gl_h)
+			return Hidden.parse_header(self,gl_h,url)
+		end
 	else
 		return nil,err
 	end
@@ -389,37 +483,55 @@ function Private.get_cookie(self,name)
 		end)
 end
 
-function Private.get_head(self,url,exhed)
+function Private.get_head(self,url,exhed,fallback)
 	local gl_b,gl_h = {},{}
 	
 	self.curl:setopt(curl.OPT_HTTPGET,1)
-	self.curl:setopt(curl.OPT_CUSTOMREQUEST,"GET")
+	self.curl:setopt(curl.OPT_CUSTOMREQUEST,"HEAD")
 	self.curl:setopt(curl.OPT_NOBODY,1)
 	
 	Hidden.build_header(self,url,exhed)
 	
 	local rc,err = Hidden.perform(self,url,gl_h,gl_b)
-
+	
 	self.curl:setopt(curl.OPT_NOBODY,0)
 
+	-- since some server do not implement it we try the last thing
+	if rc == nil and fallback then
+		gl_b,gl_h = {},{}
+		self.curl:setopt(curl.OPT_HTTPGET,1)
+		self.curl:setopt(curl.OPT_CUSTOMREQUEST,"GET")
+		Hidden.build_header(self,url,{"Range: bytes=0-1"})
+		
+		rc,err = Hidden.perform(self,url,gl_h,gl_b)
+	end
+
 	return Hidden.continue_or_return(rc,err,gl_h,
 		Private.get_head,self,Hidden.mangle_location(self,err),exhed)
 end
 
 function Private.pipe_uri(self,url,cb,exhed) 
-	local gl_b,gl_h = {},{}
+	local gl_h = {}
 	
 	self.curl:setopt(curl.OPT_HTTPGET,1)
 	self.curl:setopt(curl.OPT_CUSTOMREQUEST,"GET")
 	
 	Hidden.build_header(self,url,exhed)
-	
 	local rc,err = Hidden.perform(self,url,gl_h,cb)
 
 	return Hidden.continue_or_return(rc,err,{""},
 		Private.pipe_uri,self,err,exhed)
 end
 
+function Private.pipe_uri_with_header(self,url,cb_h,cb_b,exhed) 
+	
+	self.curl:setopt(curl.OPT_HTTPGET,1)
+	self.curl:setopt(curl.OPT_CUSTOMREQUEST,"GET")
+	
+	Hidden.build_header(self,url,exhed)
+	return Hidden.perform(self,url,cb_h,cb_b)
+end
+
 function Private.show(self)
 	print("browser:\n\tcookies:")
 	table.foreach(self.cookies,function(_,c)
@@ -531,7 +643,6 @@ function Private.verbose_mode(self)
 	self.curl:setopt(curl.OPT_VERBOSE,1)
 end
 
-
 browser = {}
 
 --<==========================================================================>--
@@ -539,14 +650,15 @@ browser = {}
 ---
 -- Creates a new object.
 -- @return object.
-function browser.new()
+function browser.new(override_useragent)
 	local b = {
 		cookies = {},
 		referrer = false, --nil will break the metatable check
 		curl = false,
 		proxy = os.getenv("LUA_HTTP_PROXY"),
 		proxyauth = os.getenv("LUA_HTTP_PROXYAUTH"),
-		useragent = os.getenv("LUA_HTTP_USERAGENT"),
+		useragent = override_useragent or 
+		  os.getenv("LUA_HTTP_USERAGENT"),
 		fpat = os.getenv("LUA_FORCE_PROXY_AUTH_TYPE"),
 		followRefreshHeader = false,
 	}
