@@ -19,7 +19,7 @@
 	- CURLINFO ???
 	  
  ******************************************************************************
-   $Id: curl_lua.c,v 1.19 2004/07/26 17:52:17 gareuselesinge Exp $
+   $Id: curl_lua.c,v 1.23 2005/03/28 10:25:52 gareuselesinge Exp $
  ******************************************************************************/ 
 #include <lua.h>
 #include <lauxlib.h>
@@ -36,9 +36,10 @@
 
 /* think more if this means unicity... maybe store in the bag some pointers */
 /* this need that a has size > bigger_offset */
-#define CURL_WRITECB_OFF(a)	((void*)(((unsigned int)a)+0))
-#define CURL_READCB_OFF(a)	((void*)(((unsigned int)a)+1))
-#define CURL_HEADCB_OFF(a)	((void*)(((unsigned int)a)+2))
+
+#define CURL_WRITECB_OFF(a)	((void*)(((char*)a)+0))
+#define CURL_READCB_OFF(a)	((void*)(((char*)a)+1))
+#define CURL_HEADCB_OFF(a)	((void*)(((char*)a)+2))
 
 /* to check the curl version on the fly, this is a v >= LIBCURL_VERSION */
 #define CURL_NEWER(M,m,p) ((p + (m << 8) + (M << 16)) <= LIBCURL_VERSION_NUM)
@@ -679,6 +680,10 @@ while( lua_next(L,3) != 0 ){
 					L_error(L,
 					 "incomplete FORM, value missed");
 				forms[position].value = (char*)
+#ifdef __UWORD_TYPE
+					(__UWORD_TYPE) 
+#endif
+					//not sure this makes sense
 					luaL_checkint(L,7);
 			}break;
 						   
@@ -1181,6 +1186,30 @@ free(tmp);
 return 1;
 }
 
+/******************************************************************************
+ * curl_unescape
+ *
+ */ 
+static int luacurl_unescape(lua_State* L) {
+const char* data = luaL_checkstring(L,1);
+size_t len = lua_strlen(L,1);
+char * tmp;
+L_checknarg(L,1,"unescape wants 1 argument (string)");
+
+tmp = curl_unescape(data,len);
+
+lua_pushstring(L,tmp);
+
+#if CURL_NEWER(7,10,0)
+curl_free(tmp);
+#else
+free(tmp);
+#endif
+	
+return 1;
+}
+
+
 /******************************************************************************
  * curl_version
  *
@@ -1258,6 +1287,7 @@ return 1;
 static const struct luaL_reg curl_f [] = {
   {"easy_init",luacurl_easy_init},
   {"escape",luacurl_escape},
+  {"unescape",luacurl_unescape},
   {"version",luacurl_version},
 #if CURL_NEWER(7,10,0)
   {"version_info",luacurl_version_info},
