@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: luabind.h,v 1.2 2004/06/25 12:38:34 gareuselesinge Exp $
+ * $Id: luabind.h,v 1.3 2004/10/31 17:22:22 gareuselesinge Exp $
  * This file is part of FreePOPs (http://freepops.sf.net)                     *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -22,6 +22,13 @@ struct L_const{
 	char* name;
 	unsigned int value;
 };
+// fill an array of this this with {"NAME",value} and 
+// terminate it with {NULL,NULL}
+// and pass it to L_openconst_with_type
+struct L_Tuserdata{
+	char* name;
+	void* data;
+};
 
 // a more flexibla luaL_error
 void L_error(lua_State* L, char* msg, ...);
@@ -35,6 +42,10 @@ void L_checknarg(lua_State* L,int n,char* msg);
 // given a table on top of the stack we add all constats to it
 void L_openconst(lua_State* L,const struct L_const* t);
 
+// given a table on top of the stack we add all constats with type to it
+void L_openTconst(lua_State* L,const struct L_Tuserdata* t,const char * type);
+
+
 // since there is no luaL_checklightuserdata
 void * L_checkludata(lua_State* L,int n);
 
