@@ -146,6 +146,7 @@ Private.html_tags_plain = {
 }
 
 
+-- ------------------------------------------------------------------------- --
 Private.boundary_chars=
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890"
 
@@ -162,6 +163,7 @@ function Private.randomize_boundary()
 	return table.concat(t)
 end
 
+-- ------------------------------------------------------------------------- --
 function Private.needs_encoding(content_type)
 	if content_type ~= "text/plain" and
 	   content_type ~= "text/html" then
@@ -171,6 +173,7 @@ function Private.needs_encoding(content_type)
 	end
 end
 
+-- ------------------------------------------------------------------------- --
 function Private.content_transfer_encoding_of(content_type)
 	if not Private.needs_encoding(content_type) then
 		return "Content-Transfer-Encoding: quoted-printable\r\n"
@@ -179,12 +182,12 @@ function Private.content_transfer_encoding_of(content_type)
 	end
 end
 
+-- ------------------------------------------------------------------------- --
 Private.base64wrap = 45
 
 function Private.base64_io_slave(cb)
 	local buffer = ""
 	return function(s,len)
-		
 		buffer = buffer .. s
 		
 		local todo_table = {}
@@ -207,11 +210,13 @@ function Private.base64_io_slave(cb)
 	end
 end
 
+-- ------------------------------------------------------------------------- --
 Private.qpewrap=73
 Private.eq = string.byte("=",1)
 Private.lf = string.byte("\n",1)
 Private.cr = string.byte("\r",1)
 
+-- ------------------------------------------------------------------------- --
 ---
 -- Encodes the message for mail transfer.
 -- must be
@@ -231,6 +236,7 @@ function mimer.quoted_printable_encode(s)
 	return table.concat(out)
 end
 
+-- ------------------------------------------------------------------------- --
 function Private.qpr_eval_expansion(s)
 	local count = 0
 	local to = 0
@@ -238,7 +244,7 @@ function Private.qpr_eval_expansion(s)
 	local lf = Private.lf
 	local cr = Private.cr
 	
-	for i=0,string.len(s) do
+	for i=1,string.len(s) do
 		local b = string.byte(s,i)
 
 		--FIXME not perfect if "...\r" trunk "\n..."
@@ -268,6 +274,8 @@ function Private.qpr_eval_expansion(s)
 	return false,false,string.len(s)
 end
 
+-- ------------------------------------------------------------------------- --
+-- a callback that implements the "quoted printable" encoding
 function Private.quoted_printable_io_slave(cb)
 	local buffer = ""
 	return function(s,len)
@@ -301,13 +309,110 @@ function Private.quoted_printable_io_slave(cb)
 
 end
 
-function Private.attach_it(browser,boundary,send_cb)
+-- ------------------------------------------------------------------------- --
+-- wrapper for the NEW and OLD implementation
+function Private.attach_it(browser,boundary,send_cb,inlineids)
+	-- switch here between the old and tested implementation and
+	-- the new and more efficient hack
+	return Private.attach_it_new(browser,boundary,send_cb,inlineids)
+	--return Private.attach_it_old(browser,boundary,send_cb)
+end
+
+-- ------------------------------------------------------------------------- --
+-- This is the NEW implementation. 
+-- 
+-- PRO:  + only one HTTP request, the header callback sets content_type and the
+--         body callback chooses on the fly the io slave.
+--       + no mere HEAD (sometimes not supported by CGIs)
+--       + acts as a browser
+-- CONS: - the code is harder
+--       - worse HTTP header parsing, no error detection. May find the 
+--         404 HTML page attached in your mail if the URL is wrong
+--       - more cpu intense, one check and a function call more than before
+--       
+function Private.attach_it_new(browser,boundary,send_cb,inlineids)	
 	return function(k,uri)
-		local h,err = browser:get_head(uri)
+		
+		-- the 2 callbacks and the shared variable content_type
+		local cb_h,cb_b = nil,nil
+		local content_type = nil
+		
+		-- the header parser, simply sets the content_type variable
+		cb_h = function(h,len)
+			-- FIXME, may be an incorrect URL and not a 200 HTTP
+			
+			-- try to extract the content type
+			local _,_,x = string.find(h or "",
+		"[Cc][Oo][Nn][Tt][Ee][Nn][Tt]%-[Tt][Yy][Pp][Ee]%s*:%s*([^\r]*)")
+			if x ~= nil then
+				content_type = x
+			end
+			return len
+		end
+		
+		-- a static variable for the callback that contains the real
+		-- io slave callback
+		local real_cb = nil
+		cb_b = function(s,len)
+			-- the first time we choos the encoding depending on 
+			-- the content_type shared variable set by the cb_h
+			if real_cb == nil then
+				content_type = content_type or 
+					"application/octet-stream"
+				if Private.needs_encoding(content_type) then
+					real_cb = Private.
+					  base64_io_slave(send_cb)
+				else
+					real_cb = Private.
+					  quoted_printable_io_slave(send_cb)
+				end
+				-- we send the mime header
+				local inlineid = inlineids[k]
+				if (inlineid == nil) then
+					send_cb("--"..boundary.."\r\n"..
+						"Content-Type: "..content_type.."\r\n"..
+						"Content-Disposition: attachment; "..
+						"filename=\""..k.."\"\r\n"..
+						Private.content_transfer_encoding_of(
+							content_type)..
+						"\r\n")
+				else
+					send_cb("--"..boundary.."\r\n"..
+						"Content-Type: "..content_type.."\r\n"..
+						"Content-ID: <"..inlineid..">\r\n"..
+						"Content-Disposition: inline; "..
+						"filename=\""..k.."\"\r\n"..
+						Private.content_transfer_encoding_of(
+							content_type)..
+						"\r\n")
+				end
+			end
+			-- we simply use the real io slave
+			return real_cb(s,len)
+		end
+	
+		-- do the work
+		browser:pipe_uri_with_header(uri,cb_h,cb_b)
 
-		--print(h)
+		-- flush last bytes
+		cb_b("",0)
+	end
+end
 
-		local _,_,x = string.find(h,
+-- ------------------------------------------------------------------------- --
+-- this is the OLD implementation
+--
+-- PRO:  + safe and tested
+--       + less cpu intensive
+-- CONS: - more HTTP requests than a real browser
+--       - if HEAD is not supported a GET is done
+--       
+function Private.attach_it_old(browser,boundary,send_cb)	
+	return function(k,uri)
+
+		local h,err = browser:get_head(uri,{},true)
+
+		local _,_,x = string.find(h or "",
 		"[Cc][Oo][Nn][Tt][Ee][Nn][Tt]%-[Tt][Yy][Pp][Ee]%s*:%s*([^\r]*)")
 
 		x = x or "application/octet-stream"
@@ -323,7 +428,7 @@ function Private.attach_it(browser,boundary,send_cb)
 		if Private.needs_encoding(x) then
 			cb = Private.base64_io_slave(send_cb)
 		else
-			cb = quoted_printable_io_slave(cb)
+			cb = Private.quoted_printable_io_slave(send_cb)
 		end
 	
 		-- do the work
@@ -334,11 +439,12 @@ function Private.attach_it(browser,boundary,send_cb)
 	end
 end
 
+-- ------------------------------------------------------------------------- --
 function Private.send_alternative(body,body_html,send_cb)
 	local boundary = Private.randomize_boundary()
 	local rc = nil
 	
-	rc = send_cb("MIME-Version: 1.0 (produced by FreePOPS/MIMER)\r\n"..
+	rc = send_cb("MIME-Version: 1.0 (produced by FreePOPs/MIMER)\r\n"..
 		"Content-Type: Multipart/alternative; "..
 			"boundary=\""..boundary.."\"\r\n"..
 		"\r\n")
@@ -363,11 +469,13 @@ end
 
 
 
+-- ------------------------------------------------------------------------- --
 function Private.token_of(c)
 	local _,_,x,y = string.find(c,"^%s*([/!]?)%s*(%a+)%s*")
 	return (x or "") .. (y or "")
 end
 
+-- ------------------------------------------------------------------------- --
 function Private.html2txt(s,base_uri,html_coded,html_tags,all)
 	s = string.gsub(s,"&(%a-);",function(c)
 		c = string.lower(c)
@@ -415,8 +523,11 @@ end
 -- @param send_cb function the callback to send the message, 
 --        may be called more then once and may return not nil to stop 
 --        the ending process.
-function mimer.pipe_msg(headers,body,body_html,base_uri,attachments,browser,send_cb)
+-- @param inlineids table a table { ["filename"] = "content-Ids" } which contains the
+--        ids for inline attachments.
+function mimer.pipe_msg(headers,body,body_html,base_uri,attachments,browser,send_cb,inlineids)
 	attachments = attachments or {}
+        inlineids = inlineids or {}
 	local rc = nil
 
 	if body == nil and body_html == nil then
@@ -428,10 +539,15 @@ function mimer.pipe_msg(headers,body,body_html,base_uri,attachments,browser,send
 
 	if table.getn(attachments) > 0 then
 		local boundary = Private.randomize_boundary()
+
+		local cType = "Multipart/Mixed"
+		if table.getn(inlineids) > 0 then
+			cType = "Multipart/Related"
+		end
 		
 		local mime = "MIME-Version: 1.0 "..
 				"(produced by FreePOPS/MIMER)\r\n"..
-			"Content-Type: Multipart/Related; "..
+				"Content-Type: " .. cType .. "; "..
 				"boundary=\""..boundary.."\"\r\n"
 		
 		-- send headers
@@ -466,7 +582,7 @@ function mimer.pipe_msg(headers,body,body_html,base_uri,attachments,browser,send
 		end
 	
 		rc = table.foreach(attachments,
-			Private.attach_it(browser,boundary,send_cb))
+			Private.attach_it(browser,boundary,send_cb,inlineids))
 		if rc ~= nil then return end
 		
 		-- close message
@@ -621,4 +737,22 @@ function mimer.remove_lines_in_proper_mail_header(s,p)
 	return table.concat(result)
 end
 
+---
+-- Transforms a classical callback f(s,len) to a mimer compliant callback.
+--@param f function a function that takes s,len and returns len,error.
+--@return function a callback that returns non nil to stop (instead of 
+--                 0,"" or nil,"").
+function mimer.callback_mangler(f) 
+	return function(s)
+		local b,err = f(s,string.len(s))
+		if b == 0 or b == nil then
+			if b == nil then
+				log.error_print(err or "bad callback?")
+			end
+			return true
+		else
+			return nil
+		end
+	end
+end
 
