@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: altsocklib.c,v 1.6 2004/07/26 17:52:17 gareuselesinge Exp $
+ * $Id: altsocklib.c,v 1.8 2005/03/28 10:25:52 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -318,7 +318,11 @@ return sd;
 int sockinfo(int sd, char *info)
 {
     struct sockaddr_in sin;
+#ifndef WIN32
+    socklen_t len = sizeof(sin);
+#else
     int len = sizeof(sin);
+#endif
     memset(&sin, 0, sizeof(sin));
     if (getsockname(sd, (struct sockaddr *) &sin, &len) == -1) {
 		sockerror("sockinfo");
