@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: winsystray.c,v 1.5 2004/09/24 17:18:39 gareuselesinge Exp $
+ * $Id: winsystray.c,v 1.6 2004/09/30 18:25:38 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -66,13 +66,14 @@ HIDDEN pthread_attr_t att;
 
 
 // helper function for get_news()
+/*
 HIDDEN long str_to_num(char* v)
 {
 int a,b,c;
 sscanf(v,"%d.%d.%d",&a,&b,&c);
 return c + b*100 + a*10000;
 }
-
+*/
 /**********************
  * the real callback
  *
@@ -83,8 +84,6 @@ HIDDEN LRESULT CALLBACK wndProc(HWND hwnd, UINT message,
 POINT pt;
 HMENU hmenu, hpopup;
 NOTIFYICONDATA nid;
-char *news_msg;
-int news_out;
 
 switch (message) 
 	{
