@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: freepops.c,v 1.16 2004/09/17 18:45:08 gareuselesinge Exp $
+ * $Id: freepops.c,v 1.22 2005/01/17 21:17:37 gareuselesinge Exp $
  * This file is part of FreePOPs (http://freepops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -82,7 +82,7 @@ gid_t gid;
 #endif
 
 /*** usage ********************************************************************/
-#define GETOPT_STRING "b:p:P:A:c:u:t:l:s:dhVvwkx:"
+#define GETOPT_STRING "b:p:P:A:c:u:t:l:s:dhVvwknx:"
 HIDDEN  struct option opts[] = { { "bind", required_argument, NULL, 'b' },
 				 { "port", required_argument, NULL, 'p' },
 				 { "proxy", required_argument, NULL, 'P' },
@@ -98,11 +98,13 @@ HIDDEN  struct option opts[] = { { "bind", required_argument, NULL, 'b' },
 				 { "daemonize", no_argument, NULL, 'd' },
 				 { "suid", required_argument, NULL, 's' },
 				 { "kill", no_argument, NULL, 'k' },
+				 { "no-pid-file", no_argument, NULL, 'n' },
 				 { "toxml", required_argument, NULL, 'x' },
 				 { "force-proxy-auth-type", 
 					 required_argument, NULL, 1000 },
 				 { "fpat", 
 					 required_argument, NULL, 1000 },
+				 { "no-icon",no_argument, NULL, 1001},
 	                         { NULL, 0, NULL, 0 } };
 
 void usage(const char *progname) {
@@ -120,9 +122,13 @@ void usage(const char *progname) {
 "\t\t\t[-l|--logmode (syslog|filename|stdout)]\n"
 "\t\t\t[-x|--toxml pluginfile]\n"
 "\t\t\t[--fpat|--force-proxy-auth-type (basic|digest|ntlm|gss)]\n"
+#if defined(WIN32)
+"\t\t\t[--no-icon]\n"
+#endif
 #if !defined(WIN32) && !defined(BEOS)			
 "\t\t\t[-s|--suid user.group]\n"
 "\t\t\t[-k|--kill]\n"
+"\t\t\t[-n|--no-pid-file]\n"
 #endif
 "        %s\t[-V|--version]\n"
 "        %s\t[-h|--help]\n\n", progname, progname, progname);
@@ -348,8 +354,28 @@ lua_State* l;
 start_logging(strdup(LOGFILE),0);
 l = luabox_genbox(LUABOX_FULL);
 bootstrap(NULL,l,username,1);
+
+#ifdef WIN32
+	{
+	int len = strlen(filename) + 5;
+	char *outname = calloc(len,sizeof(char));
+	if ( outname == NULL) {
+		fprintf(stderr,"Unable to malloc\n");
+	}
+	
+	snprintf(outname,len,"%s.xml",filename);
+	if(stdout != freopen(outname,"w",stdout)) 
+		fprintf(stderr,"Unable to redirect stdout to %s\n",outname);	
+	}
+#endif
+
 luay_call(l,"s","plugins2xml.main",filename);
 lua_close(l);
+
+#ifdef WIN32
+fclose(stdout);
+#endif
+
 return 0;
 }
 
@@ -367,8 +393,13 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 	struct in_addr address;
 	char *useragent = NULL, *proxy = NULL, *proxyauth = NULL, *fpat = NULL;
 
+#if defined(WIN32)	
+	int tray_icon = 1;
+#endif
+
 #if !(defined(WIN32) && !defined(CYGWIN)) && !defined(BEOS)	
         pid_t this_pid;
+	int no_pid = 0; /* by default we want the pid file */
 #endif
 
 #if defined(WIN32) && !defined(CYGWIN)
@@ -485,6 +516,10 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 		} else if (res == 'w') {
 			/* --veryverbose */
 			verbose_output+=2;
+	#if defined(WIN32)
+		} else if (res == 1001) {
+			tray_icon = 0;		
+	#endif
 		} else if (res == 1000) {
 			/* --fpat */
 			free(fpat);
@@ -518,6 +553,9 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 		} else if (res == 's') {
 			/* --suid */
 			parse_suid(optarg);
+		} else if (res == 'n') {
+			/* --no-pid-file */
+			no_pid = 1;
 	#endif			
 		} else {
 			usage(argv[0]);
@@ -543,15 +581,17 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 #if !(defined(WIN32) && !defined(CYGWIN)) && ! defined(BEOS)
 	if(daemonize)
 		daemonize_process();
-	
-	create_pid_file(PIDFILE);
+
+	if ( ! no_pid ) 
+		create_pid_file(PIDFILE);
 	
 	set_signals();
 #endif
 
 /*** INITIALIZATION WIN32 ***/
 #if defined(WIN32) && !defined(CYGWIN)
-	create_tray_icon(hInstance,hPrevInstance,lpszCmdLine,nCmdShow);
+	if(tray_icon)
+		create_tray_icon(hInstance,hPrevInstance,lpszCmdLine,nCmdShow);
 #endif
 
 /*** GO! ***/
