@@ -7,9 +7,9 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.0.1"
+PLUGIN_VERSION = "0.0.7"
 PLUGIN_NAME = "aol.com"
-PLUGIN_REQUIRE_VERSION = "0.0.15"
+PLUGIN_REQUIRE_VERSION = "0.0.21"
 PLUGIN_LICENSE = "GNU/GPL"
 PLUGIN_URL = "http://freepops.sourceforge.net/download.php?file=aol.lua"
 PLUGIN_HOMEPAGE = "http://freepops.sourceforge.net/"
@@ -17,8 +17,13 @@ PLUGIN_AUTHORS_NAMES = {"Russell Schwager"}
 PLUGIN_AUTHORS_CONTACTS = {"russells (at) despammed (.) com"}
 PLUGIN_DOMAINS = {
 "@aol.com","@aol.com.ar","@aol.fr","@aol.com.mx","@aol.com.au","@aol.de",
-"@aol.com.pr","@aol.com.br","@jp.aol.com","@aol.com.uk","@aol.ca","@aola.com"} 
-PLUGIN_PARAMETERS = {}
+"@aol.com.pr","@aol.com.br","@jp.aol.com","@aol.com.uk","@aol.ca","@aola.com", 
+"@netscape.net"} 
+PLUGIN_PARAMETERS = 
+	{name="folder", description={
+		it=[[La cartella che vuoi ispezionare. Quella di default &egrave; Inbox. Gli altri valori possibili sono: Junk, Trash, Draft, Sent.]],
+		en=[[The folder you want to interact with. Default is New (AOL)/Inbox (Netscape), other values are for AOL: Old, Sent, Saved, Spam and Deleted and Netscape: Sent, Trash, Draft.]]}
+	}
 PLUGIN_DESCRIPTIONS = {
 	it=[[
 Per usare questo plugin dovrete usare il vostro indirizzo email completo come 
@@ -30,15 +35,13 @@ and your real password as the password.]]
 }
 
 
-
 -- TODO
--- - Netscape mail support
 -- - User defined folder
 --
 
 -- Domains supported:  aol.com, aol.com.ar, aol.fr, aol.com.mx, aol.com.au,
 --                     aol.de, aol.com.pr, aol.com.br, jp.aol.com, aol.co.uk,
---                     aol.ca, aola.com
+--                     aol.ca, aola.com, netscape.net
 
 -- ************************************************************************** --
 --  Global Strings
@@ -46,17 +49,21 @@ and your real password as the password.]]
 
 local globals = {
   -- Server URL
-  --
-  strLoginUrl = "http://my.screenname.aol.com/_cqr/login/login.psp?siteId=atlasaol&authLev=2&seamless=novl&siteState=",
+  -- 
+  strLoginUrl = "http://my.screenname.aol.com/_cqr/login/login.psp?seamless=novl&sitedomain=beta.webmail.aol.com&lang=en&locale=us&authLev=2&siteState=",
 
   -- Login strings
   --
   strLoginPostData = "screenname=%s&password=%s",
   strLoginFailed = "Login Failed - Invalid User name and/or password",
 
+  -- Logout
+  --
+  strLogout = "http://%s/logout.aspx",
+
   -- Expressions to pull out of returned HTML from Hotmail corresponding to a problem
   --
-  strRetLoginGoodLogin = '(URL=/msglist)',
+  strRetLoginGoodLogin = 'gTargetHost = "([^"]+)"';
   strRetLoginSessionNotExpired = "(mail session has expired)",
   
   -- Regular expression to extract the mail server
@@ -64,41 +71,66 @@ local globals = {
   
   -- Pattern to extract the URL to go to get the login form
   --
-  strLoginPageParamsPattern='goToLoginUrl.*mcRedir."([^"]+)"',
+  strLoginPageParamsPattern='goToLoginUrl.-Redir."([^"]+)"',
 
   -- Pattern to pull out the url's we need to go to set some cookies.
   --
   strLoginRetUrlPattern1='LANGUAGE="JavaScript" SRC="(http[^"]*)"',
+  
+  -- Pattern to find the next page of messages
+  --
+  strMsgListPrevPagePattern = '<A HREF="([^"]+)" CLASS="msglistbottomnav">Next</A>',
 
   -- Extract the server to post the login data to
   --
-  strLoginPostUrlPattern1='[Nn][Aa][Mm][Ee]="[^"]*" [Aa][Cc][Tt][Ii][Oo][Nn]="([^"]*)"',
+  strLoginPostUrlPattern1='[Mm][Ee][Tt][Hh][Oo][Dd]="[^"]*" [Aa][Cc][Tt][Ii][Oo][Nn]="([^"]*)"',
   strLoginPostUrlPattern2='[Tt][Yy][Pp][Ee]="[Hh][Ii][Dd][Dd][Ee][Nn]" [Nn][Aa][Mm][Ee]="([^"]*)" [Vv][Aa][Ll][Uu][Ee]="([^"]*)"',
-  strLoginPostUrlPattern3='[Nn][Aa][Mm][Ee]="[^"]*" METHOD="POST" [Aa][Cc][Tt][Ii][Oo][Nn]="([^"]*)"',
+  strLoginPostUrlPattern3='[Nn][Aa][Mm][Ee]="[^"]*" [Mm][Ee][Tt][Hh][Oo][Dd]="POST" [Aa][Cc][Tt][Ii][Oo][Nn]="([^"]*)"',
   
+  -- Extract the AOL internal id for the user
+  --
+  strUserIdPattern = "uid:([^&]+)&",
+
   -- Used by Stat to pull out the message ID and the size
   --
-  strMsgLineIDPattern = 'NAME="msguid" VALUE="([^"]+)"',
+  strMsgLinePattern = 'new parent%.MessageInfo%("([^"]+)",[^,]+,"[^,]+",[^,]+,([^,]+),',
 
   -- Defined Mailbox names - These define the names to use in the URL for the mailboxes
   --
-  strInbox = "SU5CT1g=",
-  strSpam = "U3BhbQ==",
-  strOldbox = "UkVBRA==",
-  strTrash = "U2F2ZWQ=",
-  strSent = "T1VUQk9Y",
+  strInbox = "New%20Mail", 
+  strSpamAOL = "Spam",
+  strOldboxAOL = "Old%20Mail",
+  strTrashAOL = "Recently%20Deleted",
+  strSentAOL = "Sent%20Mail",
+  StrSavedAOL = "Saved%20Mail",
+  
+  strTrashNetscape = "VHJhc2g=",
+  strSentNetscape = "U2VudA==",
+  strDraftNetscape = "RHJhZnQ=",
 
   strInboxPat = "([Nn]ew)",
   strOldboxPat = "([Oo]ld)",
   strSpamPat = "([Ss]pam)",
   strSentPat = "([Ss]ent)",
-  strTrashPat = "([Dd]eleted)",
+  strDeletedPat = "([Dd]eleted)",
+  strTrashPat = "([Tt]rash)",
+  strDraftPat = "([Dd]raft)",
+  strSavedPat = "([Ss]aved)",
+
+  -- Base part of a custom folder name
+  --
+  strCustomFolderBase = "Saved%2F",
 
   -- Command URLS
   --
-  strCmdMsgList = "http://%s/msglist.adp?folder=%s&start=1",
-  strCmdDelete = "http://%s/msglist.adp?folder=%s&start=1&cmd=deletemsgs&cmdnum=%s", --&msguid=X",
-  strCmdMsgView = "http://%s/attachment/%s/%s/",
+  strCmdMsgList = "http://%s//GetMessageList.aspx?user=%s&page=%d&folder=%s&previousFolder=&stateToken=&newMailToken=&version=_SRV_1_0_0_11984_",
+  strCmdDelete = "http://%s/rpc_messages.aspx?folder=%s&action=delete&user=%s&version=_SRV_1_0_0_11984_", --&uid=X",
+  strCmdMsgView = "http://%s/rfc822.aspx?user=%s&folder=%s&uid=%s",
+
+  -- Site IDs
+  --
+  strAOLID = "atlasaol",
+  strNetscapeID = "nscpenusmail",
 }
 
 -- ************************************************************************** --
@@ -113,7 +145,9 @@ internalState = {
   browser = nil,
   strMailServer = nil,
   strDomain = nil,
-  strMBox = nil
+  strMBox = nil,
+  strSiteId = "",
+  strUserId = nil,
 }
 
 -- ************************************************************************** --
@@ -160,9 +194,9 @@ function loginAOL()
   -- Define some local variables
   --
   local username = internalState.strUser
-  local password = internalState.strPassword
+  local password = curl.escape(internalState.strPassword)
   local domain = internalState.strDomain
-  local url = globals.strLoginUrl
+  local url = string.format(globals.strLoginUrl, internalState.strSiteId)
   local xml = globals.strFolderQry
   local browser = internalState.browser
 	
@@ -198,9 +232,19 @@ function loginAOL()
   -- don't run javascript and thus must do the work here of pulling out the URL that
   -- the javascript would redirect too.
   _, _, url = string.find(body, globals.strLoginPageParamsPattern)
-  url = "http://" .. browser:wherearewe() .. url
+  if (url == nil) then
+    log.error_print("Unable to figure out the redirect on the login page.")
+    return POPSERVER_ERR_UNKNOWN
+  end
   body, err = browser:get_uri(url)
 
+  -- Shouldn't happen but you never know
+  --
+  if body == nil then
+    log.error_print(globals.strLoginFailed)
+    return POPSERVER_ERR_NETWORK
+  end
+
   -- We are now at the signin page.  Let's pull out the action of the signin form and
   -- all the hidden variables.  When done, we'll post the data along with the user and password.
   -- 
@@ -214,36 +258,34 @@ function loginAOL()
       postdata = name .. "=" .. value 
     end
   end
-
   postdata = postdata .. "&" .. 
     string.format(globals.strLoginPostData, username, password)
+  url = "http://" .. browser:wherearewe() .. url
   body, err = browser:post_uri(url, postdata)
 
   -- This is where things get a little hokey.  AOL returns a page with three javascript
   -- links that need to be "GET'ed" and then a form that needs to be submitted.  We don't
   -- care at all about the results of the get's other than the cookies...YUM!.
   for value in string.gfind(body, globals.strLoginRetUrlPattern1) do
-    _, err = browser:get_uri(value)
-  end
-
-  _, _, url = string.find(body, globals.strLoginPostUrlPattern3)
-  local postdata = nil
-  local name, value  
-  for name, value in string.gfind(body, globals.strLoginPostUrlPattern2) do
-    if postdata ~= nil then
-      postdata = postdata .. "&" .. name .. "=" .. value  
-    else
-      postdata = name .. "=" .. value 
+    local body2, err2 = browser:get_uri(value)
+    local _, _, cval = string.find(body2, 'hl0ckVal="([^"]+)"')
+    if (cval ~= nil) then
+      browser:add_cookie(value, "MC_CMP_ESKX=" .. cval .."; domain=.aol.com; path=/")
+    end
+    _, _, cval = string.find(body2, 'hckVal="([^"]+)"')
+    if (cval ~= nil) then
+      browser:add_cookie(value, "MC_CMP_ESK=" .. cval .. "; domain=.aol.com; path=/")
     end
   end
-  body, err = browser:post_uri(url, postdata)
 
-  -- Shouldn't happen but you never know
+  -- Need to redirect
   --
-  if body == nil then
+  _, _, url = string.find(body, "checkErrorAndSubmitForm%([^,]+, [^,]+, '([^']+)'")
+  if url == nil then
     log.error_print(globals.strLoginFailed)
-    return POPSERVER_ERR_AUTH
+    return POPSERVER_ERR_AUTH  
   end
+  body, err = browser:get_uri(url)
 
   -- We should be logged in now! Let's check and make sure.
   --
@@ -255,11 +297,24 @@ function loginAOL()
   
   -- Save the mail server
   --
-  internalState.strMailServer = browser:wherearewe()
+  internalState.strMailServer = str
 
   -- DEBUG Message
   --
   log.dbg("AOL/Netscape Server: " .. internalState.strMailServer .. "\n")
+
+  -- Get UserID from cookie
+  --
+  local cookie = browser:get_cookie('Auth')
+  if cookie == nil then 
+    log.error_print("Unable to determine AOL internal user id.  The plugin needs to be updated.")
+  else
+    _, _, internalState.strUserId = string.find(cookie.value, globals.strUserIdPattern)
+    if internalState.strUserId == nil then 
+      log.error_print("Unable to determine AOL internal user id.  The plugin needs to be updated.")
+    end
+  end
+
   
   -- Note that we have logged in successfully
   --
@@ -289,8 +344,7 @@ function downloadMsg(pstate, msg, nLines, data)
   --
   local browser = internalState.browser
   local uidl = get_mailmessage_uidl(pstate, msg)
-  
-  local url = string.format(globals.strCmdMsgView, internalState.strMailServer,
+  local url = string.format(globals.strCmdMsgView, internalState.strMailServer, internalState.strUserId,
     internalState.strMBox, uidl);
 
   -- Debug Message
@@ -325,6 +379,10 @@ function downloadMsg(pstate, msg, nLines, data)
   -- 
   local f, _ = browser:pipe_uri(url, cb)
 
+  -- To be safe, add a blank line
+  --
+  popserver_callback("\r\n\0", data)
+
   return POPSERVER_ERR_OK
 end
 
@@ -339,19 +397,6 @@ function downloadMsg_cb(cbInfo, data)
       return 0, nil
     end
   
-    -- Clean up the end of line and other things.  In the HTML portion of a message
-    -- AOL encodes it with quoted-printable.  Let's fix the easy stuff here.  This will
-    -- probably need to be looked at again.  I am sure it will cause issues in the non-HTML
-    -- portions of the message.
-    --
-    body = string.gsub(body, "(Content%-Transfer%-Encoding:) quoted%-printable", "%1 7bit")
-    body = string.gsub(body, "=\r\n", "")
-    body = string.gsub(body, "=\n", "")
-    body = string.gsub(body, "=\r", "")
-    body = string.gsub(body, "=[Aa]0", "\n")
-    body = string.gsub(body, "=20", " ")
-    body = string.gsub(body, "=3[dD]", "=")
-
     -- Perform our "TOP" actions
     --
     if (cbInfo.nLinesRequested ~= -2) then
@@ -399,6 +444,14 @@ function user(pstate, username)
   internalState.strDomain = domain
   internalState.strUser = user
 
+  -- Set the site id
+  -- 
+  if domain == "netscape.net" then
+    internalState.strSiteId = globals.strNetscapeID
+  else
+    internalState.strSiteId = globals.strAOLID
+  end
+
   -- Get the folder
   --
   local mbox = (freepops.MODULE_ARGS or {}).folder
@@ -409,19 +462,35 @@ function user(pstate, username)
 
   local _, _, start = string.find(mbox, globals.strSpamPat)
   if start ~= nil then
-    internalState.strMBox = globals.strSpam
+    internalState.strMBox = globals.strSpamAOL
     return POPSERVER_ERR_OK
   end
 
   _, _, start = string.find(mbox, globals.strSentPat)
   if start ~= nil then
-    internalState.strMBox = globals.strSent
+    if internalState.strSiteId == globals.strAOLID then
+      internalState.strMBox = globals.strSentAOL
+    else
+      internalState.strMBox = globals.strSentNetscape
+    end
+    return POPSERVER_ERR_OK
+  end
+
+  _, _, start = string.find(mbox, globals.strDeletedPat)
+  if start ~= nil then
+    internalState.strMBox = globals.strTrashAOL
     return POPSERVER_ERR_OK
   end
 
   _, _, start = string.find(mbox, globals.strTrashPat)
   if start ~= nil then
-    internalState.strMBox = globals.strTrash
+    internalState.strMBox = globals.strTrashNetscape
+    return POPSERVER_ERR_OK
+  end
+
+  _, _, start = string.find(mbox, globals.strDraftPat)
+  if start ~= nil then
+    internalState.strMBox = globals.strDraftNetscape
     return POPSERVER_ERR_OK
   end
 
@@ -431,14 +500,15 @@ function user(pstate, username)
     return POPSERVER_ERR_OK
   end
 
-  -- TODO - set the other mailbox here and find it
-  -- when we log in.
-  -- 
+  _, _, start = string.find(mbox, globals.strSavedPat)
+  if start ~= nil then
+    internalState.strMBox = globals.strSavedAOL
+    return POPSERVER_ERR_OK
+  end
 
-  -- Defaulting to the inbox
-  --
-  log.say("Unable to figure out the mailbox specified.  Defaulting to the Inbox.\n")
-  internalState.strMBox = globals.strInbox
+  -- It's a custom folder
+  -- 
+  internalState.strMBox = globals.strCustomFolderBase .. mbox
   return POPSERVER_ERR_OK
 end
 
@@ -517,18 +587,16 @@ function quit_update(pstate)
   local browser = internalState.browser
   local cnt = get_popstate_nummesg(pstate)
   local dcnt = 0
-  local cmdCookie = browser:get_cookie('COMMAND')
 
   local cmdUrl = string.format(globals.strCmdDelete, internalState.strMailServer, 
-    internalState.strMBox, cmdCookie.value)
+    internalState.strMBox, internalState.strUserId)
   local baseUrl = cmdUrl
-  
 
   -- Cycle through the messages and see if we need to delete any of them
   -- 
   for i = 1, cnt do
     if get_mailmessage_flag(pstate, i, MAILMESSAGE_DELETE) then
-      cmdUrl = cmdUrl .. "&msguid=" .. get_mailmessage_uidl(pstate, i) 
+      cmdUrl = cmdUrl .. "&uid=" .. get_mailmessage_uidl(pstate, i) 
       dcnt = dcnt + 1
 
       -- Send out in a batch of 5
@@ -558,14 +626,19 @@ function quit_update(pstate)
     end
   end
 
-  -- Save and then Free up the session
+  -- Log out
   --
-  session.save(hash(), serialize_state(), session.OVERWRITE)
+  cmdUrl = string.format(globals.strLogout, internalState.strMailServer)
+  browser:get_uri(cmdUrl)
+
+  -- AOL acts retarded if we save the session.  We'll remove it and
+  -- force the browser to log out.
+  --
+  session.remove(hash())
   session.unlock(hash())
 
-  log.dbg("Session saved - Account: " .. internalState.strUser .. 
+  log.dbg("Session removed - Account: " .. internalState.strUser .. 
     "@" .. internalState.strDomain .. "\n")
-
   return POPSERVER_ERR_OK
 end
 
@@ -583,8 +656,8 @@ function stat(pstate)
   -- 
   local browser = internalState.browser
   local nMsgs = 0
-  local cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
-    internalState.strMBox);
+  local cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer, internalState.strUserId,
+    1, internalState.strMBox);
   local baseUrl = cmdUrl
 
   -- Debug Message
@@ -598,17 +671,13 @@ function stat(pstate)
   -- Local function to process the list of messages, getting id's and sizes
   --
   local function funcProcess(body)
-    -- Cycle through the items and store the msg id.  AOL doesn't list the size
-    -- We'll just return the same value for every message.
-    for uidl in string.gfind(body, globals.strMsgLineIDPattern) do
-      -- Hard Code the size.
-      --
-      local size = 4096
-
+    -- Cycle through the items and store the msg id and size.  
+    ---
+    for uidl, size in string.gfind(body, globals.strMsgLinePattern) do
       -- Save the information
       --
       nMsgs = nMsgs + 1
-      log.dbg("Processed STAT - Msg: " .. nMsgs .. ", UIDL: " .. uidl .. ", (hard coded) Size: " .. size)
+      log.dbg("Processed STAT - Msg: " .. nMsgs .. ", UIDL: " .. uidl .. ", Size: " .. size)
       set_popstate_nummesg(pstate, nMsgs)
       set_mailmessage_size(pstate, nMsgs, size)
       set_mailmessage_uidl(pstate, nMsgs, uidl)
@@ -618,10 +687,19 @@ function stat(pstate)
   end 
 
   -- Local Function to check for more pages of messages.  AOL lists all
-  -- its messages on one page.
+  -- its messages on one page (but Netscape does in pages of 25)
   --
   local function funcCheckForMorePages(body) 
-    return true
+    -- Look in the body and see if there is a link for a previous page
+    -- If so, change the URL
+    --
+    local _, _, nextURL = string.find(body, globals.strMsgListPrevPagePattern)
+    if nextURL ~= nil then
+      cmdUrl = "http://" .. internalState.strMailServer .. "/" .. nextURL
+      return false
+    else
+      return true
+    end
   end
 
   -- Local Function to get the list of messages
@@ -663,7 +741,7 @@ function stat(pstate)
       --
       browser = internalState.browser
       cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
-        internalState.strMBox);
+        internalState.strMBox)
 
       -- Retry to load the page
       --
