@@ -1,7 +1,7 @@
 -- ************************************************************************** --
 --  FreePOPs @gmail.com webmail interface
 --  
---  $Id: gmail.lua,v 1.26 2004/09/25 10:41:16 gareuselesinge Exp $
+--  $Id: gmail.lua,v 1.30 2004/12/18 17:52:13 gareuselesinge Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Rami Kattan <rkattan at gmail (single dot) com>
@@ -10,7 +10,7 @@
 -- these are used in the init function
 PLUGIN_VERSION = "0.0.39"
 PLUGIN_NAME = "GMail.com"
-PLUGIN_REQUIRE_VERSION = "0.0.15"
+PLUGIN_REQUIRE_VERSION = "0.0.16"
 PLUGIN_LICENSE = "GNU/GPL"
 PLUGIN_URL = "http://freepops.sourceforge.net/download.php?file=gmail.lua"
 PLUGIN_HOMEPAGE = "http://freepops.sourceforge.net/"
@@ -26,14 +26,14 @@ Le cartelle standard disponibili sono inbox, starred, sent, all, spam, trash.
 Questo &egrave; un esempio di uno user name per leggere la cartella starred:<br/>
 foo@gmail.com?folder=starred<br/>
 <br/>
-Se hai creato delle label, puoi accedervi usando il parametro ?label=nome]],
+Se hai creato delle label personalizzate, puoi accedervi usando il parametro ?label=nome]],
 		en = [[
 Used for selecting the folder to operate on (inbox is the default one).<br/>
 The standard folders are: inbox, starred, sent, all, spam, trash.<br/>
 Here is an example of a username to get the email from the starred folder:<br/>
 foo@gmail.com?folder=starred<br/>
 <br/>
-If you created personalized labels in gmail, you can access them using
+If you created custom labels in gmail, you can access them using
 the label parameter label=name.]],
 		}	
 	},
@@ -53,13 +53,13 @@ foo@gmail.com?label=Friends]],
 		it = [[
 Valori possibili:<br/>
 - export: esporta la rubrica di gmail in un file chiamato 
-gmail_contacts_export.csv che verr generato nella tua home (Unix)
+gmail_contacts_export.csv che verr&agrave; generato nella vostra home (Unix)
 o nella directory Documenti (Windows), 
-che pu&ograve; essere importato nel tuo mail client preferito.]],
+che pu&ograve; essere importato nel vostro mail client preferito.]],
 		en = [[
 Possible values:<br/>
 - export: Exports your gmail contacts into a file called 
-gmail_contacts_export.csv that will be saved in you home (Unix)
+gmail_contacts_export.csv that will be saved in your home (Unix)
 or in the My Documents directory (Windows), that can be imported 
 into your email client.]],
 		}
@@ -67,21 +67,20 @@ into your email client.]],
 }
 PLUGIN_DESCRIPTIONS = {
 	it=[[
-Questo plugin vi per mette di leggere le mail che avete in una 
+Questo plugin vi permette di leggere le mail che avete in una 
 mailbox @gmail.com.<br/>
 Per usare questo plugin dovete usare il vostro indirizzo email completo come
 user name e la vostra password reale come password.<br/>
-Aggiungendo dei parametri all username si puo scaricare la posta dalle diverse
-cartelle o label, ed anche esportare la rubrica in formatto CSV.<br/>
-Controllare la sezione "Parametri supportati" per maggiore informazione sui
+Aggiungendo dei parametri allo username si pu&ograve; scaricare la posta dalle diverse
+cartelle o label, ed anche esportare la rubrica in formato CSV.<br/>
+Controllare la sezione &quot;Parametri supportati&quot; per maggiore informazione sui
 parametri disponibili.<br/>
 <br/>
 Nota:<br/>
-Quando il cliente di posta cancella dei messagi (perche e' stato configurato per
-cancellare i messaggi dal server [dopo x giorni]), se sei nella cartella inbox
-i messaggi saranno spostati nell'archivio (cartella "all"), se sei nella cartella
-spam i messaggi saranno spostati nel cestiono (cartella trash), altrimenti i messaggi
-vengono segnati come letti.]],
+Quando il client di posta cancella dei messaggi (perche &egrave; stato configurato per
+cancellare i messaggi dal server [dopo x giorni]), se avete controllato la cartella inbox
+i messaggi saranno spostati nell'archivio (cartella "all"), se avete controllato la cartella
+spam i messaggi saranno spostati nel cestino (cartella "trash"), altrimenti saranno solo segnati come letti.]],
 	en=[[
 This is the webmail support for @gmail.com mailboxes.<br/>
 To use this plugin you have to use your full email address as the user name 
@@ -91,10 +90,10 @@ email from different folder and/or labels, and export the contacts in CSV format
 Check "Supported parameters" for more details about the available parameters.<br/>
 <br/>
 Note:<br/>
-When the email client issue the command to delete some messages (because in its
-options it is set to delete messages from server [after x days]), if you are in the
-inbox folder, email will be moved to the archive (folder "all"), if you are in the
-spam folder, email will be moved to the trash folder, else it will only be marked
+When the email client issues the command to delete some messages (because in its
+options it is set to delete messages from the server [after x days]), if checked the
+inbox folder, the email will be moved to the archive (folder "all"), while if you checked the
+spam folder, the email will be moved to the trash folder, else it will only be marked
 as read.]]
 }
 
@@ -119,7 +118,8 @@ local gmail_string = {
 	homepage="http://gmail.google.com/gmail",
 	view_email="http://gmail.google.com/gmail?view=om&th=%s&zx=%s",
 	-- message list (regexp)
-	email_stat = ',%["(%w-)",(%d),(%d),".-","([^"]-)",.-%d%]\n',
+	-- email_stat = ',%["(%w-)",(%d),(%d),".-","([^"]-)",.-%d%]\n',
+	email_stat = ',%["(%w-)",(%d),(%d),"(.-)"%]\n',
 	-- next 2 lines: link to view a message in html format,
 	-- and regexp to extract sub messages.
 	view_email_thread="http://gmail.google.com/gmail?"..
@@ -290,6 +290,10 @@ function gmail_login()
 	end
 	--print(f)
 	local _,_,uri = string.find(f,'(CheckCookie[^%"]*)"')
+	if uri == nil then
+		log.error_print(e)
+		return POPSERVER_ERR_UNKNOWN
+	end
 	uri = "http://" .. b:wherearewe() .. "/accounts/" .. uri
 	
 	--local retrive_f = support.retry_n(
@@ -800,12 +804,14 @@ function retr(pstate,msg,data)
 
 		-- tell the browser to pipe the uri using cb
 		local f,rc = b:pipe_uri(uri,cb)
-
+		
 		if not f then
 			log.error_print("Asking for "..uri.."\n")
 			log.error_print(rc.."\n")
 			return POPSERVER_ERR_NETWORK
 		else
+			popserver_callback("\r\n",data)
+			
 -- TODO: after sending the message to the client, we need to set it as read
 --       already done, but check if all is ok....
 			uri = string.format(gmail_string.msg_mark, folder)
