@@ -7,7 +7,7 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.0.3"
+PLUGIN_VERSION = "0.0.9a"
 PLUGIN_NAME = "hotmail.com"
 PLUGIN_REQUIRE_VERSION = "0.0.15"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -16,25 +16,31 @@ PLUGIN_HOMEPAGE = "http://freepops.sourceforge.net/"
 PLUGIN_AUTHORS_NAMES = {"Russell Schwager"}
 PLUGIN_AUTHORS_CONTACTS = {"russells (at) despammed (.) com"}
 PLUGIN_DOMAINS = {"@hotmail.com","@msn.com","@webtv.com","@charter.com",
-	"@compaq.net","@passport.com"}
+	"@compaq.net","@passport.com", "@hotmail.de", "@hotmail.it", "@hotmail.co.uk", 
+        "@hotmail.co.jp"}
 PLUGIN_PARAMETERS = {
 	{name="folder", description={
 		it=[[La cartella che vuoi ispezionare. Quella di default &egrave; Inbox. Gli altri valori possibili sono: Junk, Trash, Draft, Sent.]],
 		en=[[The folder you want to interact with. Default is Inbox, other values are: Junk, Trash, Draft, Sent.]]}
-	}
+	},
+	{name = "emptytrash", description = {
+		en = [[
+Parameter is used to force the plugin to empty the trash when it is done
+pulling messages.  Set the value to 1.]]
+		}	
+	},
+
 }
 PLUGIN_DESCRIPTIONS = {
 	it=[[
 Per usare questo plugin dovrete usare il vostro indirizzo email completo come 
 nome utente e la vostra vera password come password.]],
 	en=[[
-To use
-this plugin you have to use your full email address as the username
-and your real password as the password.]]
+To use this plugin you have to use your full email address as the username
+and your real password as the password.  For support, please post a question to
+the forum instead of emailing the author(s).]]
 }
 
-
-
 -- Domains supported:  hotmail.com, msn.com, webtv.com, charter.com, compaq.net,
 --                     passport.com
 
@@ -62,6 +68,7 @@ local globals = {
   --
   strRetLoginBadLogin = "(memberservices)",
   strRetLoginSessionExpired = "(Sign in)",
+  strRetStatBusy = "(form name=.hotmail.)",
   
   -- Regular expression to extract the mail server
   --
@@ -75,7 +82,7 @@ local globals = {
 
   -- Get the crumb value that is needed for every command
   --
-  strRegExpCrumb = 'curmbox=F000000001&a=([^&]*)&f',
+  strRegExpCrumb = '&a=([^"&]*)[&"]',
 
   -- Image server pattern
   --
@@ -83,24 +90,41 @@ local globals = {
 
   -- Pattern to determine if we have no messages
   --
-  strMsgListNoMsgPat = "(There are no messages in this folder)",
+  strMsgListNoMsgPat = "(<td colspan=10>)", --"(There are no messages in this folder)",
+
+  -- Pattern to determine the total number of messages
+  --
+  strMsgListCntPattern = "<td width=100. align=center>([^<]+)</td><td align=right nowrap>",
+  strMsgListCntPattern2 = "([%d]+) [MmNnBbVv][eai]",
 
   -- Used by Stat to pull out the message ID and the size
   --
-  strMsgLineLitPattern = "<tr>.*<td>[.*]{img}.*</td>.*<td>.*<img>.*</td>.*<td>[.*]{img}.*</td>.*<td>.*<input>.*</td>.*<td>.*</td>.*<td>.*<a>.*</a>.*</td>.*<td>.*</td>.*<td>.*</td>.*<td>.*</td>.*</tr>",
-  strMsgLineAbsPattern = "<O>O<O>[O]{O}O<O>O<O>O<O>O<O>O<O>[O]{O}O<O>O<O>O<X>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>X<O>O<O>",
+  strMsgLineLitPattern = ".*<tr>.*<td>[.*]{img}.*</td>.*<td>.*<img>.*</td>.*<td>[.*]{img}.*</td>.*<td>.*<input>.*</td>.*<td>.*</td>.*<td>.*<a>.*</a>.*</td>.*<td>.*</td>.*<td>.*</td>.*<td>.*</td>.*</tr>",
+  strMsgLineAbsPattern = "O<O>O<O>[O]{O}O<O>O<O>O<O>O<O>O<O>[O]{O}O<O>O<O>O<X>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>X<O>O<O>",
 
   -- Pattern used by Stat to get the next page in the list of messages
   --
-  strMsgListNextPagePattern = '<a href="[^"]+" title="(Next Page)">',
+  strMsgListNextPagePattern = '(nextpg%.gif" border=0></a>)',
+
+  -- Pattern used to detect a bad STAT page.
+  --
+  strMsgListGoodBody = 'i%.p%.attach%.gif',
+
+  -- The amount of time that the session should time out at.
+  -- This is expressed in seconds
+  --
+  nSessionTimeout = 28800,  -- 8 hours!
 
   -- Defined Mailbox names - These define the names to use in the URL for the mailboxes
   --
+  strNewFolderPattern = "(curmbox=0)",
+  strFolderPrefix = "00000000-0000-0000-0000-000",
+
   strInbox = "F000000001",
-  strJunk = "F000000005",
+  strJunk =  "F000000005",
   strTrash = "F000000004",
   strDraft = "F000000003",
-  strSent = "F000000002",
+  strSent =  "F000000002",
 
   strInboxPat = "([iI]nbox)",
   strJunkPat = "([Jj]unk)",
@@ -110,12 +134,14 @@ local globals = {
 
   -- Command URLS
   --
-  strCmdMsgList = "http://%s/cgi-bin/HoTMaiL?a=%s&curmbox=%s&sort=Date",
+  strCmdMsgList = "http://%s/cgi-bin/HoTMaiL?a=%s&curmbox=%s",
   strCmdMsgListNextPage = "&page=%d&wo=",
   strCmdDelete = "http://%s/cgi-bin/HoTMaiL",
   strCmdDeletePost = "curmbox=%s&_HMaction=delete&wo=&SMMF=0", -- &<MSGID>=on
-  strCmdMsgView = "http://%s/cgi-bin/getmsg?msg=%s&imgsafe=y&curmbox=&s&a=&s",
+  strCmdMsgView = "http://%s/cgi-bin/getmsg?msg=%s&imgsafe=y&curmbox=%s&a=%s",
   strCmdMsgViewRaw = "&raw=0",
+  strCmdEmptyTrash = "http://%s/cgi-bin/dofolders?_HMaction=DoEmpty&curmbox=F000000004&a=%s&i=F000000004",
+  strCmdLogout = "http://%s/cgi-bin/logout",
 }
 
 -- ************************************************************************** --
@@ -132,7 +158,9 @@ internalState = {
   strImgServer = nil,
   strDomain = nil,
   strCrumb = nil,
-  strMBox = nil
+  strMBox = nil,
+  bEmptyTrash = false,
+  loginTime = nil,
 }
 
 -- ************************************************************************** --
@@ -179,7 +207,8 @@ function loginHotmail()
   -- Define some local variables
   --
   local username = internalState.strUser
-  local password = internalState.strPassword
+  local password = internalState.strPassword --curl.escape(internalState.strPassword)
+  local passwordlen = string.len(internalState.strPassword)
   local domain = internalState.strDomain
   local url = globals.strLoginUrl
   local xml = globals.strFolderQry
@@ -226,7 +255,7 @@ function loginHotmail()
   local domainWithUnderscore = string.gsub(domain, "%.", "_")
   local pattern = string.format(globals.strLoginPostUrlPattern3, domainWithUnderscore)
   _, _, url = string.find(body, pattern)
-  local padding = string.sub(globals.strLoginPaddingFull, 0, 16 - string.len(password))
+  local padding = string.sub(globals.strLoginPaddingFull, 0, 16 - passwordlen)
   postdata = string.format(globals.strLoginPostData, username, domain, password, padding)
 
   body, err = browser:post_uri(url, postdata)
@@ -287,6 +316,19 @@ function loginHotmail()
     log.dbg("Couldn't figure out the image server.  Using the mail server as a default.")
   end
 
+  -- See if we are using the new folder id's
+  --
+  _, _, str = string.find(body, globals.strNewFolderPattern)
+  if str ~= nil then
+    internalState.strMBox = globals.strFolderPrefix .. 
+      string.sub(internalState.strMBox, 2, -1) 
+    log.dbg("Hotmail - Using new folder names (" .. internalState.strMBox .. ")")
+  end
+
+  -- Note the time when we logged in
+  --
+  internalState.loginTime = os.clock();
+
   -- Note that we have logged in successfully
   --
   internalState.bLoginDone = true
@@ -342,7 +384,11 @@ function downloadMsg(pstate, msg, nLines, data)
 
     -- Lines Received - Not really used for anything
     --
-    nLinesReceived = 0
+    nLinesReceived = 0,
+
+    -- Buffer
+    --
+    strBuffer = ""
   }
 	
   -- Define the callback
@@ -352,6 +398,20 @@ function downloadMsg(pstate, msg, nLines, data)
   -- Start the download on the body
   -- 
   local f, _ = browser:pipe_uri(url, cb)
+  if not f then
+    -- An empty message.  Throw an error
+    --
+    return POPSERVER_ERR_NETWORK
+  end
+
+  -- Handle whatever is left in the buffer
+  --
+  local body = cbInfo.strBuffer
+  if (string.len(body) > 0) then
+    body = cleanupBody(body)
+    body = cbInfo.strHack:dothack(body) .. "\0"
+    popserver_callback(body, data)
+  end
 
   -- Mark the message as read
   --
@@ -370,20 +430,23 @@ function downloadMsg_cb(cbInfo, data)
     if (cbInfo.nLinesRequested ~= -2 and cbInfo.nLinesReceived == -1) then
       return 0, nil
     end
-  
-    -- The only parts we care about are within <pre>..</pre>
-    --
-    body = string.gsub(body, "<pre>[%s]*", "")
-    body = string.gsub(body, "</pre>.*", "")
 
-    -- Clean up the end of line, and replace HTML tags
+    -- Update the buffer
     --
-    body = string.gsub(body, "\n", "\r\n")
-    body = string.gsub(body, "&amp;", "&")
-    body = string.gsub(body, "&lt;", "<")
-    body = string.gsub(body, "&gt;", ">")
-    body = string.gsub(body, "&quot;", '"')
-    body = string.gsub(body, "<!%-%-%$%$imageserver%-%->", internalState.strImgServer)
+    body = cbInfo.strBuffer .. body
+    cbInfo.strBuffer = ""
+    while (string.len(body) > 6 and string.find(string.sub(body, -6), "([&<])") ~= nil) do
+      cbInfo.strBuffer = string.sub(body, -6) .. cbInfo.strBuffer
+      body = string.sub(body, 1, -7)
+    end
+
+    -- Did we reach the end of the message
+    --
+    if (string.find(body, "(</pre>)") ~= nil) then
+      cbInfo.nLinesReceived = -1;
+    end
+
+    body = cleanupBody(body)
 
     -- Perform our "TOP" actions
     --
@@ -415,6 +478,23 @@ function downloadMsg_cb(cbInfo, data)
   end
 end
 
+function cleanupBody(body)
+  -- The only parts we care about are within <pre>..</pre>
+  --
+  body = string.gsub(body, "<pre>[%s]*", "")
+  body = string.gsub(body, "</pre>.*", "")
+
+  -- Clean up the end of line, and replace HTML tags
+  --
+  body = string.gsub(body, "\n", "\r\n")
+  body = string.gsub(body, "&amp;", "&")
+  body = string.gsub(body, "&lt;", "<")
+  body = string.gsub(body, "&gt;", ">")
+  body = string.gsub(body, "&quot;", "\"")
+  body = string.gsub(body, "<!%-%-%$%$imageserver%-%->", internalState.strImgServer)
+  return body
+end
+
 -- ************************************************************************** --
 --  Pop3 functions that must be defined
 -- ************************************************************************** --
@@ -422,7 +502,6 @@ end
 -- Extract the user, domain and mailbox from the username
 --
 function user(pstate, username)
-	
   -- Get the user, domain, and mailbox
   -- TODO:  mailbox - for now, just inbox
   --
@@ -432,6 +511,15 @@ function user(pstate, username)
   internalState.strDomain = domain
   internalState.strUser = user
 
+  -- If the flag emptyTrash is set to 1 ,
+  -- the trash will be emptied on 'quit'
+  --
+  local val = (freepops.MODULE_ARGS or {}).emptytrash or 0
+  if val == "1" then
+    log.dbg("Hotmail: Trash folder will be emptied on exit.")
+    internalState.bEmptyTrash = true
+  end
+
   -- Get the folder
   --
   local mbox = (freepops.MODULE_ARGS or {}).folder
@@ -478,7 +566,6 @@ end
 -- Perform login functionality
 --
 function pass(pstate, password)
-
   -- Store the password
   --
   internalState.strPassword = password
@@ -588,6 +675,38 @@ function quit_update(pstate)
     end
   end
 
+  -- Empty the trash
+  --
+  if internalState.bEmptyTrash then
+    if internalState.strCrumb ~= '' then
+      cmdUrl = string.format(globals.strCmdEmptyTrash, internalState.strMailServer,internalState.strCrumb)
+      log.dbg("Sending Empty Trash URL: " .. cmdUrl .."\n")
+      local body, err = browser:get_uri(cmdUrl)
+      if not body or err then
+        log.error_print("Error when trying to empty the trash with url: ".. cmdUrl .."\n")
+      end
+    else
+      log.error_print("Cannot empty trash - crumb not found\n")
+    end
+  end
+
+  -- Should we force a logout.  If this session runs for more than a day, things
+  -- stop working
+  --
+  local currTime = os.clock()
+  local diff = currTime - internalState.loginTime
+  if diff > globals.nSessionTimeout then 
+    cmdUrl = string.format(globals.strCmdLogout, internalState.strMailServer)
+    log.dbg("Sending Logout URL: " .. cmdUrl .. "\n")
+    local body, err = browser:get_uri(cmdUrl)
+ 
+    log.dbg("Logout forced to keep hotmail session fresh and tasty!  Yum!\n")
+    log.dbg("Session removed - Account: " .. internalState.strUser .. 
+      "@" .. internalState.strDomain .. "\n")
+    session.remove(hash())
+    return POPSERVER_ERR_OK
+  end
+
   -- Save and then Free up the session
   --
   session.save(hash(), serialize_state(), session.OVERWRITE)
@@ -614,6 +733,7 @@ function stat(pstate)
   local browser = internalState.browser
   local nPage = 1
   local nMsgs = 0
+  local nTotMsgs = 0;
   local cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
     internalState.strCrumb, internalState.strMBox);
   local baseUrl = cmdUrl
@@ -712,14 +832,19 @@ function stat(pstate)
   -- change the command url
   --
   local function funcCheckForMorePages(body) 
-    -- Look in the body and see if there is a link for a next page
-    -- If so, change the URL
+    -- See if there are messages remaining
     --
-    local _, _, hasNextPage = string.find(body, globals.strMsgListNextPagePattern)
-    if hasNextPage ~= nil then
+    if nMsgs < nTotMsgs then
       cmdUrl = baseUrl .. string.format(globals.strCmdMsgListNextPage, nPage)
       return false
     else
+      -- For western languages, our patterns don't work so use a backup pattern.
+      --
+      if (nTotMsgs == 0 and 
+          string.find(body, globals.strMsgListNextPagePattern) ~= nil) then
+        cmdUrl = baseUrl .. string.format(globals.strCmdMsgListNextPage, nPage)
+        return false
+      end
       return true
     end
   end
@@ -738,6 +863,13 @@ function stat(pstate)
       return body, err
     end
 
+    -- Check to see if we got a good page back.  The folder list is
+    -- notorous for returning Server busy or page not available
+    --
+    if (string.find(body, globals.strRetStatBusy) == nil) then
+      return nil, "Hotmail is returning an error page."
+    end
+
     -- Is the session expired
     --
     local _, _, strSessExpr = string.find(body, globals.strRetLoginSessionExpired)
@@ -767,7 +899,31 @@ function stat(pstate)
       --
       return browser:get_uri(cmdUrl)
     end
-		
+
+    -- Get the total number of messages
+    --
+    if nTotMsgs == 0 then
+      local _, _, strTotMsgs = string.find(body, globals.strMsgListCntPattern)
+      if strTotMsgs == nil then
+        nTotMsgs = 0
+      else 
+        -- The number of messages can be in one of two patterns
+        --
+        _, _, nTotMsgs = string.find(strTotMsgs, globals.strMsgListCntPattern2)
+        if (nTotMsgs == nil) then
+          nTotMsgs = 0
+        end
+        nTotMsgs = tonumber(nTotMsgs)
+      end
+      log.dbg("Total messages in message list: " .. nTotMsgs)
+    end
+
+    -- Make sure the page is valid
+    -- 
+    if string.find(body, globals.strMsgListGoodBody) == nil then
+      return nil, "Hotmail returned with an invalid page."
+    end
+	
     return body, err
   end
 
@@ -778,13 +934,20 @@ function stat(pstate)
   if not support.do_until(funcGetPage, funcCheckForMorePages, funcProcess) then
     log.error_print("STAT Failed.\n")
     session.remove(hash())
-    return POPSERVER_ERR_UNKNOWN
+    return POPSERVER_ERR_NETWORK
   end
 	
   -- Update our state
   --
   internalState.bStatDone = true
 	
+  -- Check to see that we completed successfully.  If not, return a network
+  -- error.  This is the safest way to let the email client now that there is
+  -- a problem but that it shouldn't drop the list of known uidls.
+  if (nMsgs < nTotMsgs) then
+    return POPSERVER_ERR_NETWORK
+  end
+
   -- Return that we succeeded
   --
   return POPSERVER_ERR_OK
