@@ -1,40 +1,80 @@
 -- ************************************************************************** --
 --  FreePOPs @tre.it webmail interface
 --  
---  $Id: tre.lua,v 1.5 2004/09/14 17:08:15 gareuselesinge Exp $
+--  $Id: tre.lua,v 1.13 2005/01/06 16:21:57 dpeddi Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Eddi De Pieri <dpeddi@users.sourceforge.net>
 -- ************************************************************************** --
 
 -- these are used in the init function
-PLUGIN_VERSION = "0.0.1"
+PLUGIN_VERSION = "0.0.3"
 PLUGIN_NAME = "Tre"
-PLUGIN_REQUIRE_VERSION = "0.0.15"
+PLUGIN_REQUIRE_VERSION = "0.0.22"
 PLUGIN_LICENSE = "GNU/GPL"
 PLUGIN_URL = "http://freepops.sourceforge.net/download.php?file=tre.lua"
 PLUGIN_HOMEPAGE = "http://freepops.sourceforge.net/"
 PLUGIN_AUTHORS_NAMES = {"Eddi De Pieri"}
 PLUGIN_AUTHORS_CONTACTS = {"dpeddi (at) users (.) sourceforge (.) net"}
-PLUGIN_DOMAINS = {"@tre.it"} 
-PLUGIN_PARAMETERS = {}
+PLUGIN_DOMAINS = {"@tre.it", "@three.com.au"} 
+-- list of tables with fields name and description.
+-- description must be in the stle of PLUGIN_DESCRIPTIONS,
+-- so something like {it="bla bla bla", en = "bla bla bla"}
+PLUGIN_PARAMETERS = {
+	{name = "purge", description = {
+		it = [[
+Elimina automaticamente la posta cancellata dal cestino. Valori permessi: yes/no
+Es: 393921234567@tre.it?purge=yes ]],
+		en = [[
+Remove automatically deleted mail from trashcan. Permitted flags: yes/no
+Ie: 443921234567@three.com.au?purge=yes ]],
+		}
+	},
+        {name = "folder", description = {
+                it = [[
+Serve per selezionare la cartella (inbox &egrave; quella di default)
+su cui operare.
+Le cartelle standard disponibili sono INBOX, INBOX.Draft, INBOX.Sent, INBOX.trash.
+Se hai creato delle cartelle dalla webmail allora puoi accedervi usando il
+loro nome con il suffisso "INBOX.". es: 393921234567@tre.it?folder=INBOX.Esempio]],
+		en = [[
+Select a folder (inbox is the default folder).
+Standard folders are  INBOX, INBOX.Draft, INBOX.Sent, INBOX.trash.
+If you have creaded some your own folders you can select it using its name with 
+the "INBOX." suffix.
+ie: 443921234567@three.com.au?folder=INBOX.Example]],
+        	}
+	},
+}
+						
 PLUGIN_DESCRIPTIONS = {
 	it=[[
-Per usare questo plugin dovrete usare il vostro indirizzo email completo come 
-nome utente e la vostra vera password come password.]],
+Per usare questo plugin dovrete impostare nel vostro client di posta come
+nome utente il vostro numero di telefono nel formato 393921234567@tre.it 
+e come password il pin originale della vostra usim, indicato nella busta
+sigillata fornita da tre.]],
 	en=[[
-To use
-this plugin you have to use your full email address as the username
-and your real password as the password.]]
+To use this plugin you have to configure your mail client using as username
+your phone number formatted as 393921234567@three.com.XX and as password the usim's
+original pin code provided by three.
+PS: this plugin is tested only whith "three italy", please report me if it works
+with the three webmail of your country!.
+]]
 }
 
+-- This plugin should be finished, so I increased the version number.
+-- Perhaps there are some bugfix to do.
+
+-- The three webmail seems to be written in a very poor way!
+-- there are some wrongs with html like: <a href[...]>TEXT<a> that
+-- make parsing html more difficult!
+-- or nested comment tags...
 
 -- Todo:
--- Delete message -> ok
--- Attachment handling	-> ok
--- Select folder using params 
--- Empty trash after delete
--- Rfc-ing header date
+-- Clean lua source (Remove debug print and optimization)
+-- Since I hope three with solve issue with html, I keep at the moment all
+-- print for a near future.
+-- End
 
 -- ************************************************************************** --
 --  strings
@@ -50,7 +90,7 @@ and your real password as the password.]]
 -- 
 local tre_string = {
 	-- The uri the browser uses when you click the "login" button
-	login = "http://%s:%s@webmail.tre.it/cgi-bin/messagecenter.cgi",
+	login = "http://webmail.%s/cgi-bin/messagecenter.cgi",
 	session_errorC = "(http://[^/]+/tre/src/redirect.php)",
 	loginC = '.*<a.*href="([^"]+)".*>E.*mail.*',
 
@@ -59,30 +99,30 @@ local tre_string = {
 	statG = 'O<O>O<O>O<O>O<O><O>[O]{O}<O><O>O<O><O>O<O>O<O>O<O>O<O>O<O>O<O><O>O<O><O>O<O>O<O>O<O><X>O<O><O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>X<O>O<O>O<O>O<O><O><O><O><O>O<O>O',
 
 	-- The uri for the first page with the list of messages
-	first = "http://%s:%s@webmail.tre.it/cgi-bin/listfolders.cgi?count=5&do=viewfolder&folder=INBOX",
+	first = "http://webmail.%s/cgi-bin/listfolders.cgi?count=5&do=viewfolder&folder=%s",
 	-- The uri to get the next page of messages
 	nextC = "message%s(%d+)-(%d+),%stotal:%s(%d+)",
-	next = "http://%s:%s@webmail.tre.it/cgi-bin/listfolders.cgi?count=5&skip=%d&do=viewfolder&folder=INBOX",
+	next = "http://webmail.%s/cgi-bin/listfolders.cgi?count=5&skip=%d&do=viewfolder&folder=%s",
 
 	-- The capture to understand if the session ended
 	timeoutC = '(FIXME)',
 	-- The uri to save a message (read download the message)
-	save = 'http://%s:%s@webmail.tre.it/cgi-bin/inbox.cgi?do=viewmessage%d&folder=INBOX',
-
-	body_begin= '<td%swidth="542"%salign="left"%sclass="bodyText">',
-	body_end= '<br/></td>',
-
-	headerE = '.*<tr>.*<td><img></td>.*<td>.*</td>[.*]{!--}.*<td>[.*]{a}.*{.*a}[.*]</td>.*',
-	headerG = 'O<O>O<O><O><O>O<O>X<O>[O]{O}O<O>[O]{O}X{O}[O]<O>O',
+	save = 'http://webmail.%s/cgi-bin/inbox.cgi?do=viewmessage%d&folder=%s',
+	bodyC= '(<tr%swidth="572">.*<td%swidth="542"%salign="left"%sclass="bodyText">(.*)</td>.*</tr>)',
+--	headerE = '.*<tr>.*<td><img></td>.*<td>.*</td>[.*]{!--}.*<td>[.*]{a}.*{.*a}[.*]</td>.*',
+--	headerG = 'O<O>O<O><O><O>O<O>X<O>[O]{O}O<O>[O]{O}X{O}[O]<O>O',
+--- se <a></a> nei campi a/cc/bcc compiaono pi volte si imputtana il parser
+	headerE = '.*<tr>.*<td><img></td>.*<td>.*</td>[.*]{!--}.*<td>.*</td>.*',
+	headerG = 'O<O>O<O><O><O>O<O>X<O>[O]{O}O<O>X<O>O',
 
 	-- The uri to delete some messages
-	delete = "http://%s:%s@webmail.tre.it/cgi-bin/viewmsg.cgi",
-	delete_post = "count=14&do=delete&folder=INBOX&",
+	delete = "http://webmail.%s/cgi-bin/viewmsg.cgi",
+	delete_post = "count=14&do=delete&folder=%s&",
 	-- The peace of uri you must append to delete to choose the messages 
 	-- to delete
 	delete_next = "msgid=%s&",
 	
-	attach = 'http://%s:%s@webmail.tre.it/cgi-bin/viewmsg.cgi?do=viewattach&folder=INBOX&msgid=%s',
+	attach = 'http://webmail.%s/cgi-bin/viewmsg.cgi?do=viewattach&folder=%s&msgid=%s',
 	attachE = '<tr>.*<td></td>.*<td>.*</td>.*<td>.*</td>.*<td>.*</td>.*<td><a>.*</a></td>.*<td></td>.*</tr>.*',
 	attachG = '<O>O<O><O>O<O>O<O>O<O>X<O>O<O>O<O>O<O><X>O<O><O>O<O><O>O<O>O',
    
@@ -120,16 +160,33 @@ internal_state = {}
 --  Helpers functions
 -- ************************************************************************** --
 
+---------------------------------------------------------------------------------- Build a mail header date string
+--
+function build_date(str)
+        if(str==nil) then
+            return(os.date("%a, %d %b %Y %H:%M:%S"))
+        else
+	    -- 22/12/2004  21:10 (wrong format)
+	    -- parse date with gsub
+	    local temp
+	    str=string.gsub(str,"[0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+","%2/%1/%3 %4:%5:00")
+	    temp = getdate.toint( str )
+	    log.dbg("Date: " .. str .. " Date in UNIX format: " .. temp )
+            return(os.date("%a, %d %b %Y %H:%M:%S",temp))
+	end
+end
+
 --------------------------------------------------------------------------------
 -- Build a mail header
 --
 function build_mail_header(hfrom,hto,hcc,hdate,hsubject,uidl)
+-- Corrected way of date formatting! "16 Jan 2003 16:35:49 -0000"
         return
         "Message-Id: <"..uidl..">\r\n"..
         "From: "..hfrom.."\r\n"..
         "To: "..hto.."\r\n"..
         "Cc: "..hcc.."\r\n"..
-        "Date: "..hdate.."\r\n"..
+        "Date: "..build_date(hdate).."\r\n"..
         "Subject: "..hsubject.."\r\n"..
         "User-Agent: freepops "..PLUGIN_NAME..
         " plugin "..PLUGIN_VERSION.."\r\n"
@@ -169,7 +226,9 @@ end
 function key()
 	return (internal_state.name or "")..
 		(internal_state.domain or "")..
-		(internal_state.password or "")
+		(internal_state.password or "")..
+		(internal_state.purge or "")..
+                (internal_state.folder or "")			
 end
 
 --------------------------------------------------------------------------------
@@ -199,8 +258,8 @@ function tre_login()
 
 	-- build the uri
 	local domain = internal_state.domain
-	local uri = string.format(tre_string.login,internal_state.name,internal_state.password)
-	--print ( "DEBUG: " .. uri )
+	local uri = string.format(tre_string.login,internal_state.domain)
+	log.dbg("DEBUG: login uri " .. uri )
 
 	-- the browser must be preserved
 	internal_state.b = browser.new()
@@ -208,6 +267,7 @@ function tre_login()
 	local b = internal_state.b
 
 	--b.curl:setopt(curl.OPT_VERBOSE,1)
+	b.curl:setopt(curl.OPT_USERPWD,internal_state.name..":"..internal_state.password)
 
 	local extract_f = support.do_extract(
 		internal_state,"login_url",tre_string.loginC)
@@ -252,8 +312,9 @@ function mangle_body(s)
 	                "content%-disposition","mime%-version"})				
 
 		-- the webmail damages these tags
+		-- three webmail doesn't support html mail, so this simplify parsing!!!
 		s = mimer.remove_tags(s,
-			{"html","head","body","doctype","void","style"})
+			{"html","head","body","doctype","void","style","table","td","tr","img"})
 	
 		s = tre_string.html_preamble .. s .. 
 			tre_string.html_conclusion
@@ -276,22 +337,29 @@ function tre_parse_webmessage(pstate,msg)
 	
 	-- body handling: build the uri
 	local uidl = get_mailmessage_uidl(pstate,msg)
-	local uri = string.format(tre_string.save,internal_state.name,internal_state.password,uidl)
-
+	local uri = string.format(tre_string.save,internal_state.domain,uidl,internal_state.folder)
+	log.dbg("DEBUG: web message " .. uri )
+	
 	-- get the main mail page
 	local f,rc = b:get_uri(uri)
 
 	-- extract the body
-	local from,to = string.find(f,tre_string.body_begin)
-	local from1,to1 = string.find(f,tre_string.body_end)
+	local _,_,body = string.find(f,tre_string.bodyC)
 
-	--print ( from .. " " .. to .. " " .. from1 .. " " .. to1)
+	log.dbg("YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY")
+	log.dbg("DEBUG: Parsing message")
+	log.dbg("BODY" .. body)
+	log.dbg("YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY")
 
-	local body = string.sub(f,to+1,from1-1)
+	local subst = 1
+
+        while subst > 0 do
+	    log.dbg("DEBUG: number of substitution " .. subst )
+	    f,subst = string.gsub(f,"<[aA] href=\"[^\"]+\" [a-z]+='[^']+' [a-z]+='[^']+' [a-z]+='[^']+'>([^<]*)<[/]*a>","%1")
+	end	
 
 	-- generate the headers
 	local x = mlex.match(f,tre_string.headerE,tre_string.headerG)
-	--x:print()
 
 	local hfrom
 	local hto
@@ -301,10 +369,14 @@ function tre_parse_webmessage(pstate,msg)
 	local n = x:count()
 
 	for i = 1,n do
-		--print("addo " .. i .. " fino a " .. n)
+		log.dbg("addo " .. i .. " fino a " .. n)
 		local _,_,k = string.find(x:get(0,i-1),'([A-Za-z]+):.*')
+
+		log.dbg("DEBUG:" .. k)
+
 		local _,_,v = string.find(mimer.html2txtplain(x:get(1,i-1)),'^[%s%t]*(.*)')
 
+		log.dbg("DEBUG:" .. x:get(1,i-1))
 		if k == "Da" then
 		    hfrom = v
 		end
@@ -324,23 +396,27 @@ function tre_parse_webmessage(pstate,msg)
 
 
 	-- attach handling: build the uri
-	local uri = string.format(tre_string.attach,internal_state.name,internal_state.password,uidl)
+	local uri = string.format(tre_string.attach,internal_state.domain,internal_state.folder,uidl)
 	-- get the main attach page
 	local f,rc = b:get_uri(uri)
 
 	-- extracts the attach list
 	local x = mlex.match(f,tre_string.attachE,tre_string.attachG)
-	--x:print()
+
+	log.dbg("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")
+	log.dbg("DEBUG: extracts the attach list from " .. uri )
+	log.dbg("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")
 	
 	local n = x:count()
 	local attach = {}
 	
 	for i = 1,n do
-		--print("addo fino a " .. n)
+		log.dbg("addo fino a " .. n)
 		local _,_,url = string.find(x:get(1,i-1),'href="([^"]*)"')
 		url = string.gsub(url,"&amp;", "&")
 		local _,_,fname = string.find(x:get(0,i-1),'^[%s%t]*(.*)')
-		attach[mimer.html2txtplain(fname)] = "http://"..internal_state.name..":"..internal_state.password.."@" .. b:wherearewe() .. "/cgi-bin/" .. url
+		attach[mimer.html2txtplain(fname)] = "http://".. b:wherearewe() .. "/cgi-bin/" .. url
+		log.dbg("DEBUG: attacchment url " .. attach[mimer.html2txtplain(fname)] )
 		table.setn(attach,table.getn(attach) + 1)
 	end
 	
@@ -364,6 +440,13 @@ function user(pstate,username)
 	-- save domain and name
 	internal_state.domain = domain
 	internal_state.name = name
+
+        local f = (freepops.MODULE_ARGS or {}).folder or "INBOX"
+--        local f64 = base64.encode(f)
+--        local f64u = base64.encode(string.upper(f))
+--        internal_state.folder = f64
+        internal_state.folder = f
+--        internal_state.folder_uppercase = f64u		
 	
 	return POPSERVER_ERR_OK
 end
@@ -420,10 +503,13 @@ function quit_update(pstate)
 	local st = stat(pstate)
 	if st ~= POPSERVER_ERR_OK then return st end
 
+        local purge = (freepops.MODULE_ARGS or {}).purge or "yes"
+
 	-- shorten names, not really important
 	local b = internal_state.b
-	local uri = string.format(tre_string.delete,internal_state.name,internal_state.password)
-	local post = string.format(tre_string.delete_post)
+	local uri = string.format(tre_string.delete,internal_state.domain)
+	local post = string.format(tre_string.delete_post,internal_state.folder)
+	local post_trash = string.format(tre_string.delete_post,"INBOX.Trash")
 
 	-- here we need the stat, we build the uri and we check if we 
 	-- need to delete something
@@ -433,8 +519,10 @@ function quit_update(pstate)
 		if get_mailmessage_flag(pstate,i,MAILMESSAGE_DELETE) then
 			post = post .. string.format(tre_string.delete_next,
 				get_mailmessage_uidl(pstate,i))
-			post_trash = post .. ".Trash" .. string.format(tre_string.delete_next,
-				get_mailmessage_uidl(pstate,i))
+			if ( purge == "yes") then
+				post_trash = post_trash .. string.format(tre_string.delete_next,
+	    			    get_mailmessage_uidl(pstate,i))
+			end
 			delete_something = true	
 		end
 	end
@@ -476,9 +564,12 @@ function stat(pstate)
 	-- shorten names, not really important
 	local b = internal_state.b
 
+	b.curl:setopt(curl.OPT_USERPWD,internal_state.name..":"..internal_state.password)
+
 	-- this string will contain the uri to get. it may be updated by 
 	-- the check_f function, see later
-	local uri = string.format(tre_string.first,internal_state.name,internal_state.password)
+
+	local uri = string.format(tre_string.first,internal_state.domain,internal_state.folder)
 	
 	-- The action for do_until
 	--
@@ -541,10 +632,10 @@ function stat(pstate)
 		    return true
 --                    error("unable to capture last or to")
                 end
-		print ( "DEBUG: " .. from .. " " .. to .. " " .. last )
+--		print ( "DEBUG: " .. from .. " " .. to .. " " .. last )
 
 		if tonumber(to) < tonumber(last) then
-			uri = string.format(tre_string.next,internal_state.name,internal_state.password,to)
+			uri = string.format(tre_string.next,internal_state.domain,to,internal_state.folder)
 			-- continue the loop
 			return false
 		else
@@ -554,7 +645,7 @@ function stat(pstate)
 
 	-- this is simple and uri-dependent
 	local function retrive_f ()  
-		--print("getting "..uri)
+		log.dbg("getting "..uri)
 		local f,err = b:get_uri(uri)
 		if f == nil then
 			return f,err
@@ -573,7 +664,7 @@ function stat(pstate)
 			b = internal_state.b
 			-- popserver has not changed
 			
-			uri = string.format(tre_string.first,internal_state.name,internal_state.password)
+			uri = string.format(tre_string.first,internal_state.domain)
 
 			return b:get_uri(uri)
 		end
