@@ -8,7 +8,7 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.0.8"
+PLUGIN_VERSION = "0.1.6"
 PLUGIN_NAME = "yahoo.com"
 PLUGIN_REQUIRE_VERSION = "0.0.17"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -18,7 +18,7 @@ PLUGIN_AUTHORS_NAMES = {"Russell Schwager","Nicola Cocchiaro"}
 PLUGIN_AUTHORS_CONTACTS = 
 	{"russells (at) despammed (.) com",
          "ncocchiaro (at) users (.) sourceforge (.) net"}
-PLUGIN_DOMAINS = {"@yahoo.com","@yahoo.it", "@yahoo.ca"}
+PLUGIN_DOMAINS = {"@yahoo.com","@yahoo.it", "@yahoo.ca", "@rocketmail.com"}
 PLUGIN_PARAMETERS = {
 	{name = "folder", description = {
 		it = [[
@@ -42,7 +42,7 @@ Inviati,Anti-spam, Cestino). For user defined folders, use their name as the val
 	},
 	{name = "view", description = {
 		en = [[ Parameter is used when getting the list of messages to 
-pull.  It determines what messages to be pulled.  Possible values are ALL, UNREAD and FLAG.]]
+pull.  It determines what messages to be pulled.  Possible values are All, Unread and Flag.]]
 		}
 	},
 	{name = "markunread", description = {
@@ -50,6 +50,23 @@ pull.  It determines what messages to be pulled.  Possible values are ALL, UNREA
 pulls as unread.  If the value is 1, the behavior is turned on.]]
 		}
 	},
+	{name = "nossl", description = {
+		en = [[ Parameter is used to force the module to login through plain
+HTTP and not HTTPS with SSL.  If the value is 1, the SSL is not used.]]
+		}
+	},
+	{name = "emptytrash", description = {
+		en = [[
+Parameter is used to force the plugin to empty the trash when it is done
+pulling messages.  Set the value to 1.]]
+		}	
+	},
+	{name = "emptybulk", description = {
+		en = [[
+Parameter is used to force the plugin to empty the bulk when it is done
+pulling messages.  Set the value to 1.]]
+		}	
+	},
 }
 
 PLUGIN_DESCRIPTIONS = {
@@ -91,7 +108,8 @@ local globals = {
 
   -- Get the mail server for Yahoo
   --
-  strRegExpMailServer = '<a href="(http://[^"]*)ym/',
+  --strRegExpMailServer = '<a href="(http://[^"]*)ym/',
+  strRegExpMailServer = '(http://[^/]+/)ym/',
   
   -- Redirect site on login
   --
@@ -104,6 +122,7 @@ local globals = {
   -- Get the crumb value that is needed for deleting messages and emptying the trash
   --
   strRegExpCrumb = "=1&%.crumb=([^&]*)&",
+  strRegExpCrumb2 = "&UNR=1&.crumb=([^&]*)&",
 
   -- Pattern to determine if we have no messages.  If this is found, we have messages.
   --
@@ -140,10 +159,12 @@ local globals = {
 
   -- Command URLS
   --
-  strCmdMsgList = "%sym/ShowFolder?box=%s&Npos=%d&Nview=%s&order=up&sort=date",
+  strCmdMsgList = "%sym/ShowFolder?box=%s&Npos=%d&Nview=%s&order=up&sort=date&reset=1&Norder=up",
   strCmdMsgView = "%sym/ShowLetter?box=%s&PRINT=1&Nhead=f&toc=1&MsgId=%s&bodyPart=%s",
+  strCmdMsgWebView = "%sym/ShowLetter?box=%s&MsgId=%s",
   strCmdDelete = "%sym/ShowFolder?box=%s&DEL=Delete", -- &Mid=%s&.crumb=%s
-  strCmdEmptyTrash = "%sym/ShowFolder?ET=1&.crumb=%s&reset=1", -- &box=Inbox
+  strCmdEmptyTrash = "%sym/ShowFolder?ET=1&.crumb=%s&reset=1", 
+  strCmdEmptyBulk = "%sym/ShowFolder?EB=1&.crumb=%s&reset=1", 
   strCmdUnread = "%sym/ShowLetter?box=%s&MsgId=%s&.crumb=%s&UNR=1",
 
   -- Emails to list - These define the filter on the messages to grab
@@ -201,8 +222,85 @@ internalState = {
   strIntFlag = nil,
   strView = nil,
   bMarkMsgAsUnread = false,
+  bNoSSL = false,
+  bEmptyTrash = false,
+  bEmptyBulk = false,
+  msgids = {}
 }
 
+-- ************************************************************************** --
+--  Temporary functions for debugging and fixing CRLF bug
+-- ************************************************************************** --
+
+-- Raw Logging does not modify the given log line or data in any way:
+--   i.e. the strings are not truncated and any CR / LFs are written unchanged.
+--   The current date and time is also prefixed.
+-- 
+-- Example entry:
+--   12/05/04 03:48:17 : My Log Line
+--   --------------------------------------------------
+--   My Data
+--   --------------------------------------------------
+--
+
+-- Set to true to enable Raw Logging
+--
+local ENABLE_LOGRAW = false
+
+-- The platform dependent End Of Line string
+-- e.g. this should be changed to "\n" under UNIX, etc.
+local EOL = "\r\n"
+
+-- The raw logging function
+--
+log.raw = function ( line, data )
+  if not ENABLE_LOGRAW then
+    return
+  end
+
+  local out = assert(io.open("log_raw.txt", "ab"))
+  out:write( EOL .. os.date("%c") .. " : " )
+  out:write( line )
+  if data ~= nil then
+    out:write( EOL .. "--------------------------------------------------" .. EOL )
+    out:write( data )
+    out:write( EOL .. "--------------------------------------------------" )
+  end
+  assert(out:close())
+end
+
+-- Returns "data" with "{LF}" prefixing all \n and "{CR}" prefixing all \r,
+--   so hex viewing for those bytes isn't necessary.
+--
+function showCRLF( data )
+  local str = data
+  
+  if (str == nil) then
+    return str
+  end
+
+  str = string.gsub(str, "\n", "{LF}\n")
+  str = string.gsub(str, "\r", "{CR}\r")
+  return str
+end
+
+-- Returns "data" with all single \r and \n replaced by \r\n
+--
+function fixCRLF( data )
+  local str = data
+
+  if (str == nil) then
+    return str
+  end
+
+  -- temporarily convert proper ending to \n
+  --
+  str = string.gsub(str, "\r\n", "\n")
+  str = string.gsub(str, "\r", "\n") -- should we worry about embedded \r?
+  str = string.gsub(str, "\n", "\r\n")
+  return str
+end
+
 -- ************************************************************************** --
 --  Helper functions
 -- ************************************************************************** --
@@ -228,7 +326,8 @@ end
 function hash()
   return (internalState.strUser or "") .. "~" ..
          (internalState.strDomain or "") .. "~"  ..
-         (internalState.strMBox or "")
+         (internalState.strMBox or "") .. "~" ..
+         (internalState.strView or "")
 end
 
 -- Issue the command to login to Yahoo
@@ -236,6 +335,9 @@ end
 function loginYahoo()
   -- Check to see if we've already logged in
   --
+
+  log.raw( "Entering loginYahoo()" )
+
   if internalState.loginDone then
     return POPSERVER_ERR_OK
   end
@@ -255,11 +357,23 @@ function loginYahoo()
   local browser = internalState.browser
   local post
   local challengeCode
+
+  -- Handle rocketmail
+  --
+  if (domain == "rocketmail.com") then
+    domain = "yahoo.com"
+    username = username .. ".rm"
+  end
 	
   -- DEBUG - Set the browser in verbose mode
   --
   -- browser:verbose_mode()
 
+  if internalState.bNoSSL == true then
+    log.dbg("Yahoo: SSL login will not be used.")
+    SSLEnabled = false
+  end
+
   if SSLEnabled then
     url = globals.strLoginHTTPs
     browser:ssl_init_stuff()
@@ -324,21 +438,7 @@ function loginYahoo()
       body, err = browser:get_uri(str)
     end
   end
-  
-  -- Extract the crumb - This is needed for deletion of items
-  --
-  _, _, str = string.find(body, globals.strRegExpCrumb)
-  if str == nil then
-    log.error_print("Can't get the 'crumb' value. Will not be able to delete messages.")
-    internalState.strCrumb = ""
-  else
-    internalState.strCrumb = str
-  
-    -- Debug Message
-    -- 
-    log.dbg("Yahoo Crumb value: " .. str .. "\n")
-  end
-  
+    
   -- Note that we have logged in successfully
   --
   internalState.bLoginDone = true
@@ -358,6 +458,9 @@ end
 function downloadYahooMsg(pstate, msg, nLines, data)
   -- Make sure we aren't jumping the gun
   --
+  
+  log.raw("Entering downloadYahooMsg")
+  
   local retCode = stat(pstate)
   if retCode ~= POPSERVER_ERR_OK then 
     return retCode 
@@ -367,11 +470,15 @@ function downloadYahooMsg(pstate, msg, nLines, data)
   --
   local browser = internalState.browser
   local uidl = get_mailmessage_uidl(pstate, msg)
+  local msgid = internalState.msgids[uidl]
   
   local hdrUrl = string.format(globals.strCmdMsgView, internalState.strMailServer,
-    internalState.strMBox, uidl, "HEADER");
+    internalState.strMBox, msgid, "HEADER");
   local bodyUrl = string.format(globals.strCmdMsgView, internalState.strMailServer,
-    internalState.strMBox, uidl, "TEXT");
+    internalState.strMBox, msgid, "TEXT");
+
+  log.raw("hdrUrl = " .. hdrUrl)
+  log.raw("bodyUrl = " .. bodyUrl)
 
   -- Get the header
   --
@@ -392,6 +499,10 @@ function downloadYahooMsg(pstate, msg, nLines, data)
     --
     strHack = stringhack.new(),
 
+    -- String buffer
+    --
+    strBuffer = "",
+
     -- Lines requested (-2 means no limited)
     --
     nLinesRequested = nLines,
@@ -411,25 +522,41 @@ function downloadYahooMsg(pstate, msg, nLines, data)
 
   -- Start the download on the body
   -- 
-  local f, _ = browser:pipe_uri(bodyUrl,cb)
-  if not f then
-    -- An empty message.  Send the headers anyway
-    --
-    popserver_callback(headers, data)
-  else
-    -- Just send an extra carriage return
-    --
-    popserver_callback("\r\n\0", data)
+  if nLines ~= 0 then
+    local f, _ = browser:pipe_uri(bodyUrl,cb)
+    if not f then
+      -- An empty message.  Send the headers anyway
+      --
+      log.dbg("Empty message")
+    else
+      -- Just send an extra carriage return
+      --
+      log.dbg("Message Body has been processed.")
+      if (cbInfo.strBuffer ~= "\r\n") then
+        log.dbg("Message doesn't end in CRLF, adding to prevent client timeout.")
+        popserver_callback("\r\n\0", data)
+      end
+    end
   end
 
   -- Do we need to mark the message as unread?
   --
   if internalState.bMarkMsgAsUnread == true then
-    local cmdUrl = string.format(globals.strCmdUnread, internalState.strMailServer,
-      internalState.strMBox, uidl, internalState.strCrumb);
-    browser:get(cmdUrl) -- We don't care about the results.
+    local cmdUrl = string.format(globals.strCmdMsgWebView, internalState.strMailServer,
+      internalState.strMBox, msgid);
+    local str, _ = browser:get_uri(cmdUrl) 
+    _, _, str = string.find(str, globals.strRegExpCrumb2)
+    if str == nil then
+      log.warn("Unable to get the crumb view for marking message as unread.")
+    else
+      cmdUrl = string.format(globals.strCmdUnread, internalState.strMailServer,
+        internalState.strMBox, msgid, str);
+      browser:get_uri(cmdUrl) -- We don't care about the results.
+    end
   end
 
+  log.raw("Exiting downloadYahooMsg")
+
   return POPSERVER_ERR_OK
 end
 
@@ -438,15 +565,24 @@ end
 function downloadMsg_cb(cbInfo, data)
 	
   return function(body, len)
+
+    log.raw("Entering downloadMsg_cb generated function")
+
+    log.raw("cbInfo.nLinesRequested = " .. cbInfo.nLinesRequested)
+    log.raw("cbInfo.nLinesReceived = " .. cbInfo.nLinesReceived)
+    log.raw("cbInfo.strHack:current_lines() = " .. cbInfo.strHack:current_lines())
+
     -- Are we done with Top and should just ignore the chunks
     --
     if (cbInfo.nLinesRequested ~= -2 and cbInfo.nLinesReceived == -1) then
+      log.raw("downloadMsg_cb: return 0, nil")
       return 0, nil
     end
   
     -- Clean up the end of line
     --
-    body = string.gsub(body, "\n", "\r\n")
+    body = fixCRLF(body)
+    cbInfo.strBuffer = string.sub(body, -2, -1)
 
     -- Perform our "TOP" actions
     --
@@ -458,6 +594,7 @@ function downloadMsg_cb(cbInfo, data)
       if cbInfo.strHack:check_stop(cbInfo.nLinesRequested) then
         cbInfo.nLinesReceived = -1;
         if (string.sub(body, -2, -1) ~= "\r\n") then
+          log.error_print("Does NOT end in CRLF, adding it!")
           body = body .. "\r\n"
         end
       else
@@ -470,10 +607,14 @@ function downloadMsg_cb(cbInfo, data)
     --
     body = cbInfo.strHack:dothack(body) .. "\0"
 
+    log.raw("finished text = ", body)
+
     -- Send the data up the stream
     --
     popserver_callback(body, data)
 			
+    log.raw("Exiting downloadMsg_cb generated function")
+    
     return len, nil
   end
 end
@@ -575,8 +716,33 @@ function user(pstate, username)
   -- that we pull as unread when done.
   --
   local val = (freepops.MODULE_ARGS or {}).markunread or 0
-  if val == 1 then
-    interalState.bMarkMsgAsUnread = true
+  if val == "1" then
+    log.dbg("Yahoo: All messages pulled will be marked unread.")
+    internalState.bMarkMsgAsUnread = true
+  end
+
+  val = (freepops.MODULE_ARGS or {}).nossl or 0
+  if val == "1" then
+    log.dbg("Yahoo: SSL is disabled.")
+    internalState.bNoSSL = true
+  end
+
+  -- If the flag emptyTrash is set to 1 ,
+  -- the trash will be emptied on 'quit'
+  --
+  val = (freepops.MODULE_ARGS or {}).emptytrash or 0
+  if val == "1" then
+    log.dbg("Yahoo: Trash folder will be emptied on exit.")
+    internalState.bEmptyTrash = true
+  end
+
+  -- If the flag emptyBulk is set to 1 ,
+  -- the trash will be emptied on 'quit'
+  --
+  val = (freepops.MODULE_ARGS or {}).emptybulk or 0
+  if val == "1" then
+    log.dbg("Yahoo: Bulk folder will be emptied on exit.")
+    internalState.bEmptyBulk = true
   end
 
   return POPSERVER_ERR_OK
@@ -665,7 +831,9 @@ function quit_update(pstate)
   -- 
   for i = 1, cnt do
     if get_mailmessage_flag(pstate, i, MAILMESSAGE_DELETE) then
-      cmdUrl = cmdUrl .. "&Mid=" .. get_mailmessage_uidl(pstate, i)
+      local uidl = get_mailmessage_uidl(pstate, i)
+      local msgid = internalState.msgids[uidl]
+      cmdUrl = cmdUrl .. "&Mid=" .. msgid
       dcnt = dcnt + 1
 
       -- Send out in a batch of 5
@@ -697,6 +865,36 @@ function quit_update(pstate)
     end
   end
 
+  -- Empty the trash
+  --
+  if internalState.bEmptyTrash then
+    if internalState.strCrumb ~= '' then
+      cmdUrl = string.format(globals.strCmdEmptyTrash, internalState.strMailServer,internalState.strCrumb)
+      log.dbg("Sending Empty Trash URL: ".. cmdUrl .."\n")
+      local body, err = browser:get_uri(cmdUrl)
+      if not body or err then
+        log.error_print("Error when trying to empty the trash with url: ".. cmdUrl .."\n")
+      end
+    else
+      log.error_print("Cannot empty trash - crumb not found\n")
+    end
+  end
+
+  -- Empty the bulk folder
+  --
+  if internalState.bEmptyBulk then
+    if internalState.strCrumb ~= '' then
+      cmdUrl = string.format(globals.strCmdEmptyBulk, internalState.strMailServer,internalState.strCrumb)
+      log.dbg("Sending Empty Bulk URL: ".. cmdUrl .."\n")
+      local body, err = browser:get_uri(cmdUrl)
+      if not body or err then
+        log.error_print("Error when trying to empty the bulk with url: ".. cmdUrl .."\n")
+      end
+    else
+      log.error_print("Cannot empty bulk - crumb not found\n")
+    end
+  end
+
   -- Save and then Free up the session
   --
   session.save(hash(), serialize_state(), session.OVERWRITE)
@@ -778,22 +976,24 @@ function stat(pstate)
     -- Cycle through the items and store the msg id and size
     --
     for i = 1, cnt do
-      local uidl = items:get(0, i - 1) 
+      local msgid = items:get(0, i - 1)
       local size = items:get(1, i - 1)
 
-      if not uidl or not size then
+      if not msgid or not size then
         log.say("Yahoo Module needs to fix it's individual message list pattern matching.\n")
         return nil, "Unable to parse the size and uidl from the html"
       end
 
       -- Get the message id.  It's a series of a numbers followed by
-      -- an underscore repeated.  .
+      -- an underscore repeated.  
       --
-      _, _, uidl = string.find(uidl, globals.strMsgIDPattern) --'value="([%d-_]+)"')
+      _, _, msgid = string.find(msgid, globals.strMsgIDPattern) --'value="([%d-_]+)"')
+      local uidl = string.gsub(msgid, "_[^_]-_[^_]-_", "_000_000_", 1);
+      uidl = string.sub(uidl, 1, 60)
 
       local bUnique = true
       for j = 0, nMsgs do
-        if knownIDs[j + 1] == uidl then
+        if knownIDs[j + 1] == msgid then
           bUnique = false
           break
         end        
@@ -818,8 +1018,9 @@ function stat(pstate)
         set_popstate_nummesg(pstate, nMsgs)
         set_mailmessage_size(pstate, nMsgs, size)
         set_mailmessage_uidl(pstate, nMsgs, uidl)
-        knownIDs[nMsgs] = uidl
-      end
+        knownIDs[nMsgs] = msgid
+        internalState.msgids[uidl] = msgid
+       end
     end
 		
     return true, nil
@@ -881,6 +1082,22 @@ function stat(pstate)
       --
       return browser:get_uri(cmdUrl)
     end
+
+    -- Extract the crumb - This is needed for deletion of items
+    --
+    if (internalState.strCrumb == nil) then
+      local _, _, str = string.find(body, globals.strRegExpCrumb)
+      if str == nil then
+        log.error_print("Can't get the 'crumb' value. Will not be able to delete messages.")
+        internalState.strCrumb = ""
+      else
+        internalState.strCrumb = str
+  
+        -- Debug Message
+        -- 
+        log.dbg("Yahoo Crumb value: " .. str .. "\n")
+      end
+    end
 		
     return body, err
   end
